# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from .test_create_host import GmpCreateHostTestMixin
from .test_delete_host import GmpDeleteHostTestMixin
from .test_get_host import GmpGetHostTestMixin
from .test_get_hosts import GmpGetHostsTestMixin
from .test_modify_host import GmpModifyHostTestMixin

__all__ = (
    "GmpCreateHostTestMixin",
    "GmpDeleteHostTestMixin",
    "GmpGetHostTestMixin",
    "GmpGetHostsTestMixin",
    "GmpModifyHostTestMixin",
)



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest

from gvm.errors import RequiredArgument
from gvm.protocols.gmp.requests.v224 import TrashCan


class TrashCanTestCase(unittest.TestCase):
    def test_empty_trashcan(self):
        request = TrashCan.empty_trashcan()

        self.assertEqual(bytes(request), b"<empty_trashcan/>")

    def test_restore_from_trashcan(self):
        request = TrashCan.restore_from_trashcan("1")

        self.assertEqual(bytes(request), b'<restore id="1"/>')

    def test_restore_from_trashcan_with_empty_id(self):
        with self.assertRaises(RequiredArgument):
            TrashCan.restore_from_trashcan("")



#  SPDX-FileCopyrightText: 2025 Greenbone AG
#
#  SPDX-License-Identifier: GPL-3.0-or-later
#


class GmpGetOCIImageTargetsTestMixin:
    def test_get_targets(self):
        self.gmp.get_oci_image_targets()

        self.connection.send.has_been_called_with(b"<get_oci_image_targets/>")

    def test_get_targets_with_filter_string(self):
        self.gmp.get_oci_image_targets(filter_string="foo=bar")

        self.connection.send.has_been_called_with(
            b'<get_oci_image_targets filter="foo=bar"/>'
        )

    def test_get_targets_with_filter_id(self):
        self.gmp.get_oci_image_targets(filter_id="f1")

        self.connection.send.has_been_called_with(
            b'<get_oci_image_targets filt_id="f1"/>'
        )

    def test_get_targets_with_trash(self):
        self.gmp.get_oci_image_targets(trash=True)

        self.connection.send.has_been_called_with(
            b'<get_oci_image_targets trash="1"/>'
        )

        self.gmp.get_oci_image_targets(trash=False)

        self.connection.send.has_been_called_with(
            b'<get_oci_image_targets trash="0"/>'
        )

    def test_get_targets_with_tasks(self):
        self.gmp.get_oci_image_targets(tasks=True)

        self.connection.send.has_been_called_with(
            b'<get_oci_image_targets tasks="1"/>'
        )

        self.gmp.get_oci_image_targets(tasks=False)

        self.connection.send.has_been_called_with(
            b'<get_oci_image_targets tasks="0"/>'
        )



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224 import Gmpv224TestCase
from .authentication import (
    GmpAuthenticateTestMixin,
    GmpDescribeAuthTestMixin,
    GmpModifyAuthTestMixin,
)


class Gmpv224AuthenticateTestCase(GmpAuthenticateTestMixin, Gmpv224TestCase):
    pass


class Gmpv224ModifyAuthTestCase(GmpModifyAuthTestMixin, Gmpv224TestCase):
    pass


class Gmpv224DescribeAuthCommandTestCase(
    GmpDescribeAuthTestMixin, Gmpv224TestCase
):
    pass



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224.entities.roles import (
    GmpCloneRoleTestMixin,
    GmpCreateRoleTestMixin,
    GmpDeleteRoleTestMixin,
    GmpGetRolesTestMixin,
    GmpGetRoleTestMixin,
    GmpModifyRoleTestMixin,
)
from ...gmpv225 import GMPTestCase


class Gmpv225DeleteRoleTestCase(GmpDeleteRoleTestMixin, GMPTestCase):
    pass


class Gmpv225GetRoleTestCase(GmpGetRoleTestMixin, GMPTestCase):
    pass


class Gmpv225GetRolesTestCase(GmpGetRolesTestMixin, GMPTestCase):
    pass


class Gmpv225CloneRoleTestCase(GmpCloneRoleTestMixin, GMPTestCase):
    pass


class Gmpv225CreateRoleTestCase(GmpCreateRoleTestMixin, GMPTestCase):
    pass


class Gmpv225ModifyRoleTestCase(GmpModifyRoleTestMixin, GMPTestCase):
    pass



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#


class GmpGetScanConfigPreferencesTestMixin:
    def test_get_scan_config_preferences(self):
        self.gmp.get_scan_config_preferences()

        self.connection.send.has_been_called_with(b"<get_preferences/>")

    def test_get_scan_config_preferences_with_nvt_oid(self):
        self.gmp.get_scan_config_preferences(nvt_oid="oid")

        self.connection.send.has_been_called_with(
            b'<get_preferences nvt_oid="oid"/>'
        )

    def test_get_scan_config_preferences_with_config_id(self):
        self.gmp.get_scan_config_preferences(config_id="c1")

        self.connection.send.has_been_called_with(
            b'<get_preferences config_id="c1"/>'
        )



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import RequiredArgument


class GmpGetPermissionTestMixin:
    def test_get_permission(self):
        self.gmp.get_permission("p1")

        self.connection.send.has_been_called_with(
            b'<get_permissions permission_id="p1"/>'
        )

        self.gmp.get_permission(permission_id="p1")

        self.connection.send.has_been_called_with(
            b'<get_permissions permission_id="p1"/>'
        )

    def test_get_permission_missing_permission_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.get_permission(permission_id=None)

        with self.assertRaises(RequiredArgument):
            self.gmp.get_permission("")



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import InvalidArgument, RequiredArgument
from gvm.protocols.gmp.requests.v226 import ResourceType


class GmpGetResourceNamesListTestMixin:
    def test_get_resource_names(self):
        self.gmp.get_resource_names(ResourceType.ALERT)

        self.connection.send.has_been_called_with(
            b'<get_resource_names type="ALERT"/>'
        )

        self.gmp.get_resource_names(ResourceType.CERT_BUND_ADV)

        self.connection.send.has_been_called_with(
            b'<get_resource_names type="CERT_BUND_ADV"/>'
        )

        self.gmp.get_resource_names(ResourceType.CONFIG)

        self.connection.send.has_been_called_with(
            b'<get_resource_names type="CONFIG"/>'
        )

        self.gmp.get_resource_names(resource_type=ResourceType.CPE)

        self.connection.send.has_been_called_with(
            b'<get_resource_names type="CPE"/>'
        )

        self.gmp.get_resource_names(ResourceType.CREDENTIAL)

        self.connection.send.has_been_called_with(
            b'<get_resource_names type="CREDENTIAL"/>'
        )

        self.gmp.get_resource_names(ResourceType.CVE)

        self.connection.send.has_been_called_with(
            b'<get_resource_names type="CVE"/>'
        )

        self.gmp.get_resource_names(ResourceType.DFN_CERT_ADV)

        self.connection.send.has_been_called_with(
            b'<get_resource_names type="DFN_CERT_ADV"/>'
        )

        self.gmp.get_resource_names(ResourceType.FILTER)

        self.connection.send.has_been_called_with(
            b'<get_resource_names type="FILTER"/>'
        )

        self.gmp.get_resource_names(ResourceType.GROUP)

        self.connection.send.has_been_called_with(
            b'<get_resource_names type="GROUP"/>'
        )

        self.gmp.get_resource_names(ResourceType.HOST)

        self.connection.send.has_been_called_with(
            b'<get_resource_names type="HOST"/>'
        )

        self.gmp.get_resource_names(ResourceType.NOTE)

        self.connection.send.has_been_called_with(
            b'<get_resource_names type="NOTE"/>'
        )

        self.gmp.get_resource_names(ResourceType.NVT)

        self.connection.send.has_been_called_with(
            b'<get_resource_names type="NVT"/>'
        )

        self.gmp.get_resource_names(ResourceType.OS)

        self.connection.send.has_been_called_with(
            b'<get_resource_names type="OS"/>'
        )

        self.gmp.get_resource_names(ResourceType.OVERRIDE)

        self.connection.send.has_been_called_with(
            b'<get_resource_names type="OVERRIDE"/>'
        )

        self.gmp.get_resource_names(ResourceType.PERMISSION)

        self.connection.send.has_been_called_with(
            b'<get_resource_names type="PERMISSION"/>'
        )

        self.gmp.get_resource_names(ResourceType.PORT_LIST)

        self.connection.send.has_been_called_with(
            b'<get_resource_names type="PORT_LIST"/>'
        )

        self.gmp.get_resource_names(ResourceType.REPORT_FORMAT)

        self.connection.send.has_been_called_with(
            b'<get_resource_names type="REPORT_FORMAT"/>'
        )

        self.gmp.get_resource_names(ResourceType.REPORT)

        self.connection.send.has_been_called_with(
            b'<get_resource_names type="REPORT"/>'
        )

        self.gmp.get_resource_names(ResourceType.REPORT_CONFIG)

        self.connection.send.has_been_called_with(
            b'<get_resource_names type="REPORT_CONFIG"/>'
        )

        self.gmp.get_resource_names(ResourceType.RESULT)

        self.connection.send.has_been_called_with(
            b'<get_resource_names type="RESULT"/>'
        )

        self.gmp.get_resource_names(ResourceType.ROLE)

        self.connection.send.has_been_called_with(
            b'<get_resource_names type="ROLE"/>'
        )

        self.gmp.get_resource_names(ResourceType.SCANNER)

        self.connection.send.has_been_called_with(
            b'<get_resource_names type="SCANNER"/>'
        )

        self.gmp.get_resource_names(ResourceType.SCHEDULE)

        self.connection.send.has_been_called_with(
            b'<get_resource_names type="SCHEDULE"/>'
        )

        self.gmp.get_resource_names(ResourceType.TARGET)

        self.connection.send.has_been_called_with(
            b'<get_resource_names type="TARGET"/>'
        )

        self.gmp.get_resource_names(ResourceType.TASK)

        self.connection.send.has_been_called_with(
            b'<get_resource_names type="TASK"/>'
        )

        self.gmp.get_resource_names(ResourceType.TLS_CERTIFICATE)

        self.connection.send.has_been_called_with(
            b'<get_resource_names type="TLS_CERTIFICATE"/>'
        )

        self.gmp.get_resource_names(ResourceType.USER)

        self.connection.send.has_been_called_with(
            b'<get_resource_names type="USER"/>'
        )

        with self.assertRaises(AttributeError):
            self.gmp.get_resource_names(
                ResourceType.ALLRESOURCES  # pylint: disable=no-member
            )

    def test_get_resource_names_missing_resource_type(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.get_resource_names(resource_type=None)

        with self.assertRaises(RequiredArgument):
            self.gmp.get_resource_names(resource_type="")

        with self.assertRaises(RequiredArgument):
            self.gmp.get_resource_names("")

    def test_get_resource_names_invalid_resource_type(self):
        with self.assertRaises(InvalidArgument):
            self.gmp.get_resource_names(resource_type="foo")

    def test_get_resource_names_with_filter_string(self):
        self.gmp.get_resource_names(ResourceType.CPE, filter_string="foo=bar")

        self.connection.send.has_been_called_with(
            b'<get_resource_names type="CPE" filter="foo=bar"/>'
        )



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import unittest

from lxml import etree

from gvm.transforms import EtreeTransform


class EtreeTransformTestCase(unittest.TestCase):
    def test_transform_response(self):
        transform = EtreeTransform()
        result = transform(b"<foo/>")

        self.assertTrue(etree.iselement(result))

    def test_transform_more_complex_response(self):
        transform = EtreeTransform()
        result = transform(b'<foo id="bar"><lorem/><ipsum/></foo>')

        self.assertTrue(etree.iselement(result))
        self.assertEqual(result.tag, "foo")
        self.assertEqual(result.get("id"), "bar")
        self.assertEqual(len(result), 2)



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

from typing import Protocol, runtime_checkable


@runtime_checkable
class Request(Protocol):
    """
    A GMP Request Python Protocol

    A Request implementation may be provided by several classes
    """

    def __bytes__(self) -> bytes: ...



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import RequiredArgument


class GmpGetCertBundTestMixin:
    def test_get_cert_bund_advisory(self):
        self.gmp.get_cert_bund_advisory(cert_id="i1")

        self.connection.send.has_been_called_with(
            b'<get_info info_id="i1" type="CERT_BUND_ADV" details="1"/>'
        )

        self.gmp.get_cert_bund_advisory("i1")

        self.connection.send.has_been_called_with(
            b'<get_info info_id="i1" type="CERT_BUND_ADV" details="1"/>'
        )

    def test_get_cert_bund_advisory_missing_cert_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.get_cert_bund_advisory(cert_id="")

        with self.assertRaises(RequiredArgument):
            self.gmp.get_cert_bund_advisory("")

        with self.assertRaises(RequiredArgument):
            self.gmp.get_cert_bund_advisory(cert_id=None)



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later


from gvm._enum import Enum
from gvm.errors import InvalidArgument, RequiredArgument
from gvm.protocols.core import Request
from gvm.utils import to_bool
from gvm.xml import XmlCommand

from .._entity_id import EntityID
from ._report_formats import ReportFormatType


class AlertEvent(Enum):
    """Enum for alert event types"""

    TASK_RUN_STATUS_CHANGED = "Task run status changed"
    UPDATED_SECINFO_ARRIVED = "Updated SecInfo arrived"
    NEW_SECINFO_ARRIVED = "New SecInfo arrived"
    TICKET_RECEIVED = "Ticket received"
    ASSIGNED_TICKET_CHANGED = "Assigned ticket changed"
    OWNED_TICKET_CHANGED = "Owned ticket changed"


class AlertCondition(Enum):
    """Enum for alert condition types"""

    ALWAYS = "Always"
    ERROR = "Error"
    SEVERITY_AT_LEAST = "Severity at least"
    SEVERITY_CHANGED = "Severity changed"
    FILTER_COUNT_CHANGED = "Filter count changed"
    FILTER_COUNT_AT_LEAST = "Filter count at least"


class AlertMethod(Enum):
    """Enum for alert method type"""

    SCP = "SCP"
    SEND = "Send"
    SMB = "SMB"
    SNMP = "SNMP"
    SYSLOG = "Syslog"
    EMAIL = "Email"
    START_TASK = "Start Task"
    HTTP_GET = "HTTP Get"
    SOURCEFIRE_CONNECTOR = "Sourcefire Connector"
    VERINICE_CONNECTOR = "verinice Connector"
    TIPPINGPOINT_SMS = "TippingPoint SMS"
    ALEMBA_VFIRE = "Alemba vFire"


def _check_event(
    event: AlertEvent,
    condition: AlertCondition | None,
    method: AlertMethod | None,
):
    if event == AlertEvent.TASK_RUN_STATUS_CHANGED:
        if not condition:
            raise RequiredArgument(
                f"condition is required for event {event.name}"
            )

        if not method:
            raise RequiredArgument(f"method is required for event {event.name}")

        if condition not in (
            AlertCondition.ALWAYS,
            AlertCondition.FILTER_COUNT_CHANGED,
            AlertCondition.FILTER_COUNT_AT_LEAST,
            AlertCondition.SEVERITY_AT_LEAST,
            AlertCondition.SEVERITY_CHANGED,
        ):
            raise InvalidArgument(
                f"Invalid condition {condition.name} for event {event.name}"
            )
    elif event in (
        AlertEvent.NEW_SECINFO_ARRIVED,
        AlertEvent.UPDATED_SECINFO_ARRIVED,
    ):
        if not condition:
            raise RequiredArgument(
                f"condition is required for event {event.name}"
            )

        if not method:
            raise RequiredArgument(f"method is required for event {event.name}")

        if condition != AlertCondition.ALWAYS:
            raise InvalidArgument(
                f"Invalid condition {condition.name} for event {event.name}"
            )
        if method not in (
            AlertMethod.SCP,
            AlertMethod.SEND,
            AlertMethod.SMB,
            AlertMethod.SNMP,
            AlertMethod.SYSLOG,
            AlertMethod.EMAIL,
        ):
            raise InvalidArgument(
                f"Invalid method {method.name} for event {event.name}"
            )
    elif event in (
        AlertEvent.TICKET_RECEIVED,
        AlertEvent.OWNED_TICKET_CHANGED,
        AlertEvent.ASSIGNED_TICKET_CHANGED,
    ):
        if not condition:
            raise RequiredArgument(
                f"condition is required for event {event.name}"
            )

        if not method:
            raise RequiredArgument(f"method is required for event {event.name}")
        if condition != AlertCondition.ALWAYS:
            raise InvalidArgument(
                f"Invalid condition {condition.name} for event {event.name}"
            )
        if method not in (
            AlertMethod.EMAIL,
            AlertMethod.START_TASK,
            AlertMethod.SYSLOG,
        ):
            raise InvalidArgument(
                f"Invalid method {method.name} for event {event.name}"
            )


class Alerts:
    @classmethod
    def create_alert(
        cls,
        name: str,
        condition: AlertCondition,
        event: AlertEvent,
        method: AlertMethod,
        *,
        method_data: dict[str, str] | None = None,
        event_data: dict[str, str] | None = None,
        condition_data: dict[str, str] | None = None,
        filter_id: EntityID | None = None,
        comment: str | None = None,
    ) -> Request:
        """Create a new alert

        Args:
            name: Name of the new Alert
            condition: The condition that must be satisfied for the alert
                to occur; if the event is either 'Updated SecInfo arrived' or
                'New SecInfo arrived', condition must be 'Always'. Otherwise,
                condition can also be on of 'Severity at least', 'Filter count
                changed' or 'Filter count at least'.
            event: The event that must happen for the alert to occur, one
                of 'Task run status changed', 'Updated SecInfo arrived' or 'New
                SecInfo arrived'
            method: The method by which the user is alerted, one of 'SCP',
                'Send', 'SMB', 'SNMP', 'Syslog' or 'Email'; if the event is
                neither 'Updated SecInfo arrived' nor 'New SecInfo arrived',
                method can also be one of 'Start Task', 'HTTP Get', 'Sourcefire
                Connector' or 'verinice Connector'.
            condition_data: Data that defines the condition
            event_data: Data that defines the event
            method_data: Data that defines the method
            filter_id: Filter to apply when executing alert
            comment: Comment for the alert
        """
        if not name:
            raise RequiredArgument(
                function=cls.create_alert.__name__, argument="name"
            )

        if not condition:
            raise RequiredArgument(
                function=cls.create_alert.__name__, argument="condition"
            )

        if not event:
            raise RequiredArgument(
                function=cls.create_alert.__name__, argument="event"
            )

        if not method:
            raise RequiredArgument(
                function=cls.create_alert.__name__, argument="method"
            )

        if not isinstance(condition, AlertCondition):
            condition = AlertCondition(condition)
        if not isinstance(event, AlertEvent):
            event = AlertEvent(event)
        if not isinstance(method, AlertMethod):
            method = AlertMethod(method)

        _check_event(event, condition, method)

        cmd = XmlCommand("create_alert")
        cmd.add_element("name", name)

        conditions = cmd.add_element("condition", condition.value)

        if condition_data is not None:
            for key, value in condition_data.items():
                xml_data = conditions.add_element("data", value)
                xml_data.add_element("name", key)

        events = cmd.add_element("event", event.value)

        if event_data is not None:
            for key, value in event_data.items():
                xml_data = events.add_element("data", value)
                xml_data.add_element("name", key)

        methods = cmd.add_element("method", method.value)

        if method_data is not None:
            for key, value in method_data.items():
                xml_data = methods.add_element("data", value)
                xml_data.add_element("name", key)

        if filter_id:
            cmd.add_element("filter", attrs={"id": filter_id})

        if comment:
            cmd.add_element("comment", comment)

        return cmd

    @classmethod
    def modify_alert(
        cls,
        alert_id: EntityID,
        *,
        name: str | None = None,
        comment: str | None = None,
        filter_id: EntityID | None = None,
        event: AlertEvent | str | None = None,
        event_data: dict | None = None,
        condition: AlertCondition | str | None = None,
        condition_data: dict[str, str] | None = None,
        method: AlertMethod | str | None = None,
        method_data: dict[str, str] | None = None,
    ) -> Request:
        """Modify an existing alert.

        Args:
            alert_id: UUID of the alert to be modified.
            name: Name of the Alert.
            condition: The condition that must be satisfied for the alert to
                occur. If the event is either 'Updated SecInfo
                arrived' or 'New SecInfo arrived', condition must be 'Always'.
                Otherwise, condition can also be on of 'Severity at least',
                'Filter count changed' or 'Filter count at least'.
            condition_data: Data that defines the condition
            event: The event that must happen for the alert to occur, one of
                'Task run status changed', 'Updated SecInfo arrived' or
                'New SecInfo arrived'
            event_data: Data that defines the event
            method: The method by which the user is alerted, one of 'SCP',
                'Send', 'SMB', 'SNMP', 'Syslog' or 'Email';
                if the event is neither 'Updated SecInfo arrived' nor
                'New SecInfo arrived', method can also be one of 'Start Task',
                'HTTP Get', 'Sourcefire Connector' or 'verinice Connector'.
            method_data: Data that defines the method
            filter_id: Filter to apply when executing alert
            comment: Comment for the alert
        """
        if not alert_id:
            raise RequiredArgument(
                function=cls.modify_alert.__name__, argument="alert_id"
            )

        cmd = XmlCommand("modify_alert")
        cmd.set_attribute("alert_id", str(alert_id))

        if name:
            cmd.add_element("name", name)

        if comment:
            cmd.add_element("comment", comment)

        if filter_id:
            cmd.add_element("filter", attrs={"id": filter_id})

        if condition:
            if not isinstance(condition, AlertCondition):
                condition = AlertCondition(condition)

            conditions = cmd.add_element("condition", condition.value)

            if condition_data is not None:
                for key, value in condition_data.items():
                    xml_data = conditions.add_element("data", value)
                    xml_data.add_element("name", key)

        if method:
            if not isinstance(method, AlertMethod):
                method = AlertMethod(method)

            methods = cmd.add_element("method", method.value)

            if method_data is not None:
                for key, value in method_data.items():
                    xml_data = methods.add_element("data", value)
                    xml_data.add_element("name", key)

        if event:
            if not isinstance(event, AlertEvent):
                event = AlertEvent(event)

            _check_event(event, condition, method)  # type: ignore

            events = cmd.add_element("event", event.value)

            if event_data is not None:
                for key, value in event_data.items():
                    xml_data = events.add_element("data", value)
                    xml_data.add_element("name", key)

        return cmd

    @classmethod
    def clone_alert(cls, alert_id: EntityID) -> Request:
        """Clone an existing alert

        Args:
            alert_id: UUID of the alert to clone from
        """
        if not alert_id:
            raise RequiredArgument(
                function=cls.clone_alert.__name__, argument="alert_id"
            )

        cmd = XmlCommand("create_alert")
        cmd.add_element("copy", str(alert_id))
        return cmd

    @classmethod
    def delete_alert(
        cls, alert_id: EntityID, *, ultimate: bool | None = False
    ) -> Request:
        """Delete an existing alert

        Args:
            alert_id: UUID of the alert to delete
            ultimate: Whether to remove entirely or to the trashcan.
        """
        if not alert_id:
            raise RequiredArgument(
                function=cls.delete_alert.__name__, argument="alert_id"
            )

        cmd = XmlCommand("delete_alert")
        cmd.set_attribute("alert_id", str(alert_id))
        cmd.set_attribute("ultimate", to_bool(ultimate))
        return cmd

    @classmethod
    def test_alert(cls, alert_id: EntityID) -> Request:
        """Run an alert

        Invoke a test run of an alert

        Args:
            alert_id: UUID of the alert to be tested
        """
        if not alert_id:
            raise RequiredArgument(
                function=cls.test_alert.__name__, argument="alert_id"
            )

        cmd = XmlCommand("test_alert")
        cmd.set_attribute("alert_id", str(alert_id))
        return cmd

    @classmethod
    def trigger_alert(
        cls,
        alert_id: EntityID,
        report_id: EntityID,
        *,
        filter_string: str | None = None,
        filter_id: EntityID | None = None,
        report_format_id: EntityID | ReportFormatType | None = None,
        delta_report_id: EntityID | None = None,
    ) -> Request:
        """Run an alert by ignoring its event and conditions

        The alert is triggered to run immediately with the provided filtered
        report by ignoring the even and condition settings.

        Args:
            alert_id: UUID of the alert to be run
            report_id: UUID of the report to be provided to the alert
            filter: Filter term to use to filter results in the report
            filter_id: UUID of filter to use to filter results in the report
            report_format_id: UUID of report format to use                              or ReportFormatType (enum)
            delta_report_id: UUID of an existing report to compare report to.
        """
        if not alert_id:
            raise RequiredArgument(
                function=cls.trigger_alert.__name__,
                argument="alert_id argument",
            )

        if not report_id:
            raise RequiredArgument(
                function=cls.trigger_alert.__name__,
                argument="report_id argument",
            )

        cmd = XmlCommand("get_reports")
        cmd.set_attribute("report_id", str(report_id))
        cmd.set_attribute("alert_id", str(alert_id))
        cmd.add_filter(filter_string, filter_id)

        if report_format_id:
            cmd.set_attribute("format_id", str(report_format_id))

        if delta_report_id:
            cmd.set_attribute("delta_report_id", str(delta_report_id))

        return cmd

    @staticmethod
    def get_alerts(
        *,
        filter_string: str | None = None,
        filter_id: EntityID | None = None,
        trash: bool | None = None,
        tasks: bool | None = None,
    ) -> Request:
        """Request a list of alerts

        Args:
            filter: Filter term to use for the query
            filter_id: UUID of an existing filter to use for the query
            trash: True to request the alerts in the trashcan
            tasks: Whether to include the tasks using the alerts
        """
        cmd = XmlCommand("get_alerts")
        cmd.add_filter(filter_string, filter_id)

        if trash is not None:
            cmd.set_attribute("trash", to_bool(trash))

        if tasks is not None:
            cmd.set_attribute("tasks", to_bool(tasks))

        return cmd

    @classmethod
    def get_alert(
        cls, alert_id: EntityID, *, tasks: bool | None = None
    ) -> Request:
        """Request a single alert

        Arguments:
            alert_id: UUID of an existing alert
            tasks: Whether to include the tasks using the alert
        """
        cmd = XmlCommand("get_alerts")

        if not alert_id:
            raise RequiredArgument(
                function=cls.get_alert.__name__, argument="alert_id"
            )

        cmd.set_attribute("alert_id", str(alert_id))

        if tasks is not None:
            cmd.set_attribute("tasks", to_bool(tasks))

        return cmd



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later


from gvm._enum import Enum
from gvm.protocols.core import Request
from gvm.xml import XmlCommand


class HelpFormat(Enum):
    """Enum for the help format"""

    HTML = "html"
    RNC = "rnc"
    TEXT = "text"
    XML = "xml"


class Help:
    @staticmethod
    def help(
        *,
        help_format: HelpFormat | str | None = None,
        brief: bool | None = None,
    ) -> Request:
        """Get the help text

        Arguments:
            help_format: Format of of the help:
                "html", "rnc", "text" or "xml
            brief: If True help is brief
        """
        cmd = XmlCommand("help")

        help_type = ""
        if brief:
            help_type = "brief"

        cmd.set_attribute("type", help_type)

        if help_format:
            if not isinstance(help_format, HelpFormat):
                help_format = HelpFormat(help_format)

            cmd.set_attribute("format", help_format.value)

        return cmd



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import logging
import socket as socketlib
from abc import ABC, abstractmethod
from time import time
from typing import Protocol, runtime_checkable

from gvm.errors import GvmError

BUF_SIZE = 16 * 1024

DEFAULT_TIMEOUT = 60  # in seconds

logger = logging.getLogger(__name__)


@runtime_checkable
class GvmConnection(Protocol):
    """
    Python `protocol <https://docs.python.org/3/library/typing.html#typing.Protocol>`_
    for GvmConnection classes.
    """

    def connect(self) -> None:
        """Establish a connection to a remote server"""

    def disconnect(self) -> None:
        """Send data to the connected remote server

        Arguments:
            data: Data to be send to the server. Either utf-8 encoded string or
                bytes.
        """

    def send(self, data: bytes) -> None:
        """Send data to the connected remote server

        Args:
            data: Data to be send to the server as bytes.
        """

    def read(self) -> bytes:
        """Read data from the remote server

        Returns:
            data as bytes
        """

    def finish_send(self):
        """Indicate to the remote server you are done with sending data"""


class AbstractGvmConnection(ABC):
    """
    Base class for establishing a connection to a remote server daemon.

    Arguments:
        timeout: Timeout in seconds for the connection. None to
            wait indefinitely
    """

    def __init__(self, timeout: int | float | None = DEFAULT_TIMEOUT):
        self._socket: socketlib.SocketType | None = None
        self._timeout = timeout if timeout is not None else DEFAULT_TIMEOUT

    def _read(self) -> bytes:
        if self._socket is None:
            raise GvmError("Socket is not connected")

        return self._socket.recv(BUF_SIZE)

    @abstractmethod
    def connect(self) -> None:
        """Establish a connection to a remote server"""
        raise NotImplementedError

    def send(self, data: bytes) -> None:
        """Send data to the connected remote server

        Args:
            data: Data to be send to the server as bytes.
        """
        if self._socket is None:
            raise GvmError("Socket is not connected")

        self._socket.sendall(data)

    def read(self) -> bytes:
        """Read data from the remote server

        Returns:
            data as bytes
        """
        break_timeout = (
            time() + self._timeout if self._timeout is not None else None
        )

        data = self._read()

        if not data:
            # Connection was closed by server
            raise GvmError("Remote closed the connection")

        if break_timeout and time() > break_timeout:
            raise GvmError("Timeout while reading the response")

        return data

    def disconnect(self) -> None:
        """Disconnect and close the connection to the remote server"""
        try:
            if self._socket is not None:
                self._socket.close()
        except OSError as e:
            logger.debug("Connection closing error: %s", e)

    def finish_send(self):
        """Indicate to the remote server you are done with sending data"""
        if self._socket is not None:
            # shutdown socket for sending. only allow reading data afterwards
            self._socket.shutdown(socketlib.SHUT_WR)



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import InvalidArgument, RequiredArgument
from gvm.protocols.gmp.requests.v224 import PortRangeType


class GmpCreatePortRangeTestMixin:
    def test_create_port_range_missing_port_list_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.create_port_range(
                port_list_id=None,
                start=1,
                end=1234,
                port_range_type=PortRangeType.TCP,
            )

        with self.assertRaises(RequiredArgument):
            self.gmp.create_port_range(
                port_list_id="",
                start=1,
                end=1234,
                port_range_type=PortRangeType.TCP,
            )

    def test_create_port_range_missing_start(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.create_port_range(
                port_list_id="pl1",
                start=None,
                end=1234,
                port_range_type=PortRangeType.TCP,
            )

        with self.assertRaises(RequiredArgument):
            self.gmp.create_port_range(
                port_list_id="pl1",
                start="",
                end=1234,
                port_range_type=PortRangeType.TCP,
            )

    def test_create_port_range_missing_end(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.create_port_range(
                port_list_id="pl1",
                start=1,
                end=None,
                port_range_type=PortRangeType.TCP,
            )

        with self.assertRaises(RequiredArgument):
            self.gmp.create_port_range(
                port_list_id="pl1",
                start=1,
                end="",
                port_range_type=PortRangeType.TCP,
            )

    def test_create_port_range_missing_port_range_type(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.create_port_range(
                port_list_id="pl1", start=1, end=1234, port_range_type=None
            )

        with self.assertRaises(RequiredArgument):
            self.gmp.create_port_range(
                port_list_id="pl1", start=1, end=1234, port_range_type=""
            )

    def test_create_port_range_invalid_port_range_type(self):
        with self.assertRaises(InvalidArgument):
            self.gmp.create_port_range(
                port_list_id="pl1", start=1, end=1234, port_range_type="blubb"
            )

    def test_create_port_range(self):
        self.gmp.create_port_range(
            port_list_id="pl1",
            start=1,
            end=1234,
            port_range_type=PortRangeType.TCP,
        )

        self.connection.send.has_been_called_with(
            b"<create_port_range>"
            b'<port_list id="pl1"/>'
            b"<start>1</start>"
            b"<end>1234</end>"
            b"<type>TCP</type>"
            b"</create_port_range>"
        )

        self.gmp.create_port_range(
            port_list_id="pl1",
            start=1,
            end=1234,
            port_range_type=PortRangeType.UDP,
        )

        self.connection.send.has_been_called_with(
            b"<create_port_range>"
            b'<port_list id="pl1"/>'
            b"<start>1</start>"
            b"<end>1234</end>"
            b"<type>UDP</type>"
            b"</create_port_range>"
        )

        self.gmp.create_port_range(
            port_list_id="pl1",
            start="1",
            end="1234",
            port_range_type=PortRangeType.TCP,
        )

        self.connection.send.has_been_called_with(
            b"<create_port_range>"
            b'<port_list id="pl1"/>'
            b"<start>1</start>"
            b"<end>1234</end>"
            b"<type>TCP</type>"
            b"</create_port_range>"
        )

    def test_create_port_range_with_comment(self):
        self.gmp.create_port_range(
            port_list_id="pl1",
            start=1,
            end=1234,
            port_range_type=PortRangeType.TCP,
            comment="lorem",
        )

        self.connection.send.has_been_called_with(
            b"<create_port_range>"
            b'<port_list id="pl1"/>'
            b"<start>1</start>"
            b"<end>1234</end>"
            b"<type>TCP</type>"
            b"<comment>lorem</comment>"
            b"</create_port_range>"
        )



# SPDX-FileCopyrightText: 2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

"""
openvasd HTTP API version 1

High-level API interface for interacting with openvasd HTTP services via
logical modules (health, metadata, scans, etc.).
"""

from ._client import StrOrPathLike, create_openvasd_http_client
from ._health import HealthAPI
from ._metadata import MetadataAPI
from ._notus import NotusAPI
from ._scans import ScansAPI
from ._vts import VtsAPI


class OpenvasdHttpAPIv1:
    """
    High-level interface for accessing openvasd HTTP API v1 endpoints.

    This class encapsulates modular sub-APIs (health, metadata, notus, scans, vts)
    and wires them to a shared `httpx.Client` configured for secure access.

    Each sub-API provides methods for interacting with a specific openvasd domain.
    """

    def __init__(
        self,
        host_name: str,
        port: int = 3000,
        *,
        api_key: str | None = None,
        server_ca_path: StrOrPathLike | None = None,
        client_cert_paths: StrOrPathLike
        | tuple[StrOrPathLike, StrOrPathLike]
        | None = None,
        suppress_exceptions: bool = False,
    ):
        """
        Initialize the OpenvasdHttpApiV1 entry point.

        Args:
            host_name: Hostname or IP of the openvasd server (e.g., "localhost").
            port: Port of the openvasd service (default: 3000).
            api_key: Optional API key to be used for authentication.
            server_ca_path: Path to the server CA certificate (for HTTPS/mTLS).
            client_cert_paths: Path to client certificate or (cert, key) tuple for mTLS.
            suppress_exceptions: If True, suppress exceptions and return structured error
                responses. Default is False, which means exceptions will be raised.
        """
        self._client = create_openvasd_http_client(
            host_name=host_name,
            port=port,
            api_key=api_key,
            server_ca_path=server_ca_path,
            client_cert_paths=client_cert_paths,
        )

        # Sub-API modules
        self.__health = HealthAPI(
            self._client, suppress_exceptions=suppress_exceptions
        )
        self.__metadata = MetadataAPI(
            self._client, suppress_exceptions=suppress_exceptions
        )
        self.__notus = NotusAPI(
            self._client, suppress_exceptions=suppress_exceptions
        )
        self.__scans = ScansAPI(
            self._client, suppress_exceptions=suppress_exceptions
        )
        self.__vts = VtsAPI(
            self._client, suppress_exceptions=suppress_exceptions
        )

    @property
    def health(self) -> HealthAPI:
        """
        Access the health API module.

        Provides methods to check the health status of the openvasd service.
        """
        return self.__health

    @property
    def metadata(self) -> MetadataAPI:
        """
        Access the metadata API module.

        Provides methods to retrieve metadata about the openvasd service,
        including version and feed information.
        """
        return self.__metadata

    @property
    def notus(self) -> NotusAPI:
        """
        Access the Notus API module.

        Provides methods to interact with the Notus service for package-based
        vulnerability scanning.
        """
        return self.__notus

    @property
    def scans(self) -> ScansAPI:
        """
        Access the scans API module.

        Provides methods to manage and interact with vulnerability scans,
        including starting, stopping, and retrieving scan results.
        """
        return self.__scans

    @property
    def vts(self) -> VtsAPI:
        """
        Access the VTS API module.

        Provides methods to manage and interact with Vulnerability Tests
        (VTs) for vulnerability assessment.
        """
        return self.__vts



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224 import Gmpv224TestCase
from .users import (
    GmpCloneUserTestMixin,
    GmpCreateUserTestMixin,
    GmpDeleteUserTestMixin,
    GmpGetUsersTestMixin,
    GmpGetUserTestMixin,
    GmpModifyUserTestMixin,
)


class Gmpv224CloneUserTestCase(GmpCloneUserTestMixin, Gmpv224TestCase):
    pass


class Gmpv224CreateUserTestCase(GmpCreateUserTestMixin, Gmpv224TestCase):
    pass


class Gmpv224DeleteUserTestCase(GmpDeleteUserTestMixin, Gmpv224TestCase):
    pass


class Gmpv224GetUserTestCase(GmpGetUserTestMixin, Gmpv224TestCase):
    pass


class Gmpv224GetUsersTestCase(GmpGetUsersTestMixin, Gmpv224TestCase):
    pass


class Gmpv224ModifyUserTestCase(GmpModifyUserTestMixin, Gmpv224TestCase):
    pass



# SPDX-FileCopyrightText: 2023-2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224.entities.hosts import (
    GmpCreateHostTestMixin,
    GmpDeleteHostTestMixin,
    GmpGetHostsTestMixin,
    GmpGetHostTestMixin,
    GmpModifyHostTestMixin,
)
from ...gmpv227 import GMPTestCase


class GMPCreateHostTestCase(GmpCreateHostTestMixin, GMPTestCase):
    pass


class GMPDeleteHostTestCase(GmpDeleteHostTestMixin, GMPTestCase):
    pass


class GMPGetHostTestCase(GmpGetHostTestMixin, GMPTestCase):
    pass


class GMPGetHostsTestCase(GmpGetHostsTestMixin, GMPTestCase):
    pass


class GMPModifyHostTestCase(GmpModifyHostTestMixin, GMPTestCase):
    pass



#  SPDX-FileCopyrightText: 2025 Greenbone AG
#
#  SPDX-License-Identifier: GPL-3.0-or-later

import ssl
import unittest
from unittest.mock import MagicMock, patch

from gvm.protocols.http.openvasd._client import create_openvasd_http_client


class TestOpenvasdClient(unittest.TestCase):
    @patch("gvm.protocols.http.openvasd._client.Client")
    def test_init_without_tls_or_api_key(self, mock_httpx_client):
        create_openvasd_http_client("localhost")
        mock_httpx_client.assert_called_once()
        _, kwargs = mock_httpx_client.call_args
        self.assertEqual(kwargs["base_url"], "http://localhost:3000")
        self.assertFalse(kwargs["verify"])
        self.assertNotIn("X-API-KEY", kwargs["headers"])

    @patch("gvm.protocols.http.openvasd._client.Client")
    def test_init_with_api_key_only(self, mock_httpx_client):
        create_openvasd_http_client("localhost", api_key="secret")
        _, kwargs = mock_httpx_client.call_args
        self.assertEqual(kwargs["headers"]["X-API-KEY"], "secret")
        self.assertEqual(kwargs["base_url"], "http://localhost:3000")
        self.assertFalse(kwargs["verify"])

    @patch("gvm.protocols.http.openvasd._client.ssl.create_default_context")
    @patch("gvm.protocols.http.openvasd._client.Client")
    def test_init_with_mtls_tuple(
        self, mock_httpx_client, mock_ssl_ctx_factory
    ):
        mock_context = MagicMock(spec=ssl.SSLContext)
        mock_ssl_ctx_factory.return_value = mock_context

        create_openvasd_http_client(
            "localhost",
            server_ca_path="/path/ca.pem",
            client_cert_paths=("/path/cert.pem", "/path/key.pem"),
        )

        mock_ssl_ctx_factory.assert_called_once_with(
            ssl.Purpose.SERVER_AUTH, cafile="/path/ca.pem"
        )
        mock_context.load_cert_chain.assert_called_once_with(
            certfile="/path/cert.pem", keyfile="/path/key.pem"
        )
        mock_httpx_client.assert_called_once()
        _, kwargs = mock_httpx_client.call_args
        self.assertEqual(kwargs["base_url"], "https://localhost:3000")
        self.assertEqual(kwargs["verify"], mock_context)

    @patch("gvm.protocols.http.openvasd._client.ssl.create_default_context")
    @patch("gvm.protocols.http.openvasd._client.Client")
    def test_init_with_mtls_single_cert(
        self, mock_httpx_client, mock_ssl_ctx_factory
    ):
        mock_context = MagicMock(spec=ssl.SSLContext)
        mock_ssl_ctx_factory.return_value = mock_context

        create_openvasd_http_client(
            "localhost",
            server_ca_path="/path/ca.pem",
            client_cert_paths="/path/client.pem",
        )

        mock_context.load_cert_chain.assert_called_once_with(
            certfile="/path/client.pem"
        )
        _, kwargs = mock_httpx_client.call_args
        self.assertEqual(kwargs["base_url"], "https://localhost:3000")
        self.assertEqual(kwargs["verify"], mock_context)



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
import warnings

from gvm.errors import InvalidArgument, RequiredArgument


class GmpModifyScanConfigTestMixin:
    def test_modify_scan_config_invalid_selection(self):
        with self.assertRaises(InvalidArgument):
            self.gmp.modify_scan_config(config_id="c1", selection="foo")

        with self.assertRaises(InvalidArgument):
            self.gmp.modify_scan_config(config_id="c1", selection="")

    def test_modify_scan_config_missing_config_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_scan_config(config_id=None, selection="nvt_pref")

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_scan_config(config_id="", selection="nvt_pref")

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_scan_config("", selection="nvt_pref")

    def test_modify_scan_config_set_comment(self):
        # pylint: disable=invalid-name
        with warnings.catch_warnings(record=True) as w:
            warnings.simplefilter("always")

            self.gmp.modify_scan_config(
                config_id="c1", selection=None, comment="foo"
            )

            self.assertEqual(len(w), 1)
            self.assertTrue(issubclass(w[0].category, DeprecationWarning))

        self.connection.send.has_been_called_with(
            b'<modify_config config_id="c1">'
            b"<comment>foo</comment>"
            b"</modify_config>"
        )

    def test_modify_scan_config_set_nvt_pref(self):
        # pylint: disable=invalid-name
        with warnings.catch_warnings(record=True) as w:
            warnings.simplefilter("always")

            self.gmp.modify_scan_config(
                config_id="c1", selection="nvt_pref", nvt_oid="o1", name="foo"
            )

            self.assertEqual(len(w), 1)
            self.assertTrue(issubclass(w[0].category, DeprecationWarning))

        self.connection.send.has_been_called_with(
            b'<modify_config config_id="c1">'
            b"<preference>"
            b'<nvt oid="o1"/>'
            b"<name>foo</name>"
            b"</preference>"
            b"</modify_config>"
        )

    def test_modify_scan_config_set_scanner_pref(self):
        # pylint: disable=invalid-name
        with warnings.catch_warnings(record=True) as w:
            warnings.simplefilter("always")

            self.gmp.modify_scan_config(
                config_id="c1", selection="scan_pref", name="foo", value="bar"
            )

            self.assertEqual(len(w), 1)
            self.assertTrue(issubclass(w[0].category, DeprecationWarning))

        self.connection.send.has_been_called_with(
            b'<modify_config config_id="c1">'
            b"<preference>"
            b"<name>foo</name>"
            b"<value>YmFy</value>"
            b"</preference>"
            b"</modify_config>"
        )

    def test_modify_scan_config_set_nvt_selection(self):
        # pylint: disable=invalid-name
        with warnings.catch_warnings(record=True) as w:
            warnings.simplefilter("always")

            self.gmp.modify_scan_config(
                config_id="c1",
                selection="nvt_selection",
                nvt_oids=["o1"],
                family="foo",
            )

            self.assertEqual(len(w), 1)
            self.assertTrue(issubclass(w[0].category, DeprecationWarning))

        self.connection.send.has_been_called_with(
            b'<modify_config config_id="c1">'
            b"<nvt_selection>"
            b"<family>foo</family>"
            b'<nvt oid="o1"/>'
            b"</nvt_selection>"
            b"</modify_config>"
        )

    def test_modify_scan_config_set_family_selection(self):
        # pylint: disable=invalid-name
        with warnings.catch_warnings(record=True) as w:
            warnings.simplefilter("always")

            self.gmp.modify_scan_config(
                config_id="c1",
                selection="family_selection",
                families=[("foo", True, True)],
            )

            self.assertEqual(len(w), 1)
            self.assertTrue(issubclass(w[0].category, DeprecationWarning))

        self.connection.send.has_been_called_with(
            b'<modify_config config_id="c1">'
            b"<family_selection>"
            b"<growing>1</growing>"
            b"<family>"
            b"<name>foo</name>"
            b"<all>1</all>"
            b"<growing>1</growing>"
            b"</family>"
            b"</family_selection>"
            b"</modify_config>"
        )



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224.entities.users import (
    GmpCloneUserTestMixin,
    GmpCreateUserTestMixin,
    GmpDeleteUserTestMixin,
    GmpGetUsersTestMixin,
    GmpGetUserTestMixin,
    GmpModifyUserTestMixin,
)
from ...gmpv226 import GMPTestCase


class GMPCloneUserTestCase(GmpCloneUserTestMixin, GMPTestCase):
    pass


class GMPCreateUserTestCase(GmpCreateUserTestMixin, GMPTestCase):
    pass


class GMPDeleteUserTestCase(GmpDeleteUserTestMixin, GMPTestCase):
    pass


class GMPGetUserTestCase(GmpGetUserTestMixin, GMPTestCase):
    pass


class GMPGetUsersTestCase(GmpGetUsersTestMixin, GMPTestCase):
    pass


class GMPModifyUserTestCase(GmpModifyUserTestMixin, GMPTestCase):
    pass



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import InvalidArgument, RequiredArgument
from gvm.protocols.gmp.requests.v224 import CredentialFormat


class GmpGetCredentialTestMixin:
    def test_get_credential(self):
        self.gmp.get_credential("id")

        self.connection.send.has_been_called_with(
            b'<get_credentials credential_id="id"/>'
        )

    def test_get_credential_missing_credential_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.get_credential(None)

    def test_get_credential_invalid_credential_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.get_credential(credential_id=None)

        with self.assertRaises(RequiredArgument):
            self.gmp.get_credential("")

    def test_get_credential_with_credential_format(self):
        self.gmp.get_credential("id", credential_format=CredentialFormat.KEY)

        self.connection.send.has_been_called_with(
            b'<get_credentials credential_id="id" format="key"/>'
        )

        self.gmp.get_credential("id", credential_format=CredentialFormat.RPM)

        self.connection.send.has_been_called_with(
            b'<get_credentials credential_id="id" format="rpm"/>'
        )

        self.gmp.get_credential("id", credential_format=CredentialFormat.DEB)

        self.connection.send.has_been_called_with(
            b'<get_credentials credential_id="id" format="deb"/>'
        )

        self.gmp.get_credential("id", credential_format=CredentialFormat.EXE)

        self.connection.send.has_been_called_with(
            b'<get_credentials credential_id="id" format="exe"/>'
        )

        self.gmp.get_credential("id", credential_format=CredentialFormat.PEM)

        self.connection.send.has_been_called_with(
            b'<get_credentials credential_id="id" format="pem"/>'
        )

    def test_get_credential_with_invalid_credential_format(self):
        with self.assertRaises(InvalidArgument):
            self.gmp.get_credential("id", credential_format="foo")

    def test_get_credential_with_scanners(self):
        self.gmp.get_credential("id", scanners=True)

        self.connection.send.has_been_called_with(
            b'<get_credentials credential_id="id" scanners="1"/>'
        )

        self.gmp.get_credential("id", scanners=False)

        self.connection.send.has_been_called_with(
            b'<get_credentials credential_id="id" scanners="0"/>'
        )

    def test_get_credential_with_targets(self):
        self.gmp.get_credential("id", targets=True)

        self.connection.send.has_been_called_with(
            b'<get_credentials credential_id="id" targets="1"/>'
        )

        self.gmp.get_credential("id", targets=False)

        self.connection.send.has_been_called_with(
            b'<get_credentials credential_id="id" targets="0"/>'
        )



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import unittest

from gvm.errors import InvalidArgument
from gvm.protocols.gmp.requests.v225 import PermissionSubjectType


class GetPermissionSubjectTypeFromStringTestCase(unittest.TestCase):
    def test_invalid(self):
        with self.assertRaises(InvalidArgument):
            PermissionSubjectType.from_string("foo")

    def test_none_or_empty(self):
        ct = PermissionSubjectType.from_string(None)
        self.assertIsNone(ct)
        ct = PermissionSubjectType.from_string("")
        self.assertIsNone(ct)

    def test_user(self):
        ct = PermissionSubjectType.from_string("user")
        self.assertEqual(ct, PermissionSubjectType.USER)

    def test_role(self):
        ct = PermissionSubjectType.from_string("role")
        self.assertEqual(ct, PermissionSubjectType.ROLE)

    def test_group(self):
        ct = PermissionSubjectType.from_string("group")
        self.assertEqual(ct, PermissionSubjectType.GROUP)


if __name__ == "__main__":
    unittest.main()



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224.entities.report_formats import (
    GmpCloneReportFormatTestMixin,
    GmpDeleteReportFormatTestMixin,
    GmpGetReportFormatsTestMixin,
    GmpGetReportFormatTestMixin,
    GmpImportReportFormatTestMixin,
    GmpModifyReportFormatTestMixin,
    GmpVerifyReportFormatTestMixin,
)
from ...gmpv225 import GMPTestCase


class Gmpv225DeleteReportFormatTestCase(
    GmpDeleteReportFormatTestMixin, GMPTestCase
):
    pass


class Gmpv225GetReportFormatTestCase(GmpGetReportFormatTestMixin, GMPTestCase):
    pass


class Gmpv225GetReportFormatsTestCase(
    GmpGetReportFormatsTestMixin, GMPTestCase
):
    pass


class Gmpv225CloneReportFormatTestCase(
    GmpCloneReportFormatTestMixin, GMPTestCase
):
    pass


class Gmpv225ImportReportFormatTestCase(
    GmpImportReportFormatTestMixin, GMPTestCase
):
    pass


class Gmpv225ModifyReportFormatTestCase(
    GmpModifyReportFormatTestMixin, GMPTestCase
):
    pass


class Gmpv225VerifyReportFormatTestCase(
    GmpVerifyReportFormatTestMixin, GMPTestCase
):
    pass



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224 import Gmpv224TestCase
from .filters import (
    GmpCloneFilterTestMixin,
    GmpCreateFilterTestMixin,
    GmpDeleteFilterTestMixin,
    GmpGetFiltersTestMixin,
    GmpGetFilterTestMixin,
    GmpModifyFilterTestMixin,
)


class Gmpv224DeleteFilterTestCase(GmpDeleteFilterTestMixin, Gmpv224TestCase):
    pass


class Gmpv224GetFilterTestCase(GmpGetFilterTestMixin, Gmpv224TestCase):
    pass


class Gmpv224GetFiltersTestCase(GmpGetFiltersTestMixin, Gmpv224TestCase):
    pass


class Gmpv224CloneFilterTestCase(GmpCloneFilterTestMixin, Gmpv224TestCase):
    pass


class Gmpv224CreateFilterTestCase(GmpCreateFilterTestMixin, Gmpv224TestCase):
    pass


class Gmpv224ModifyFilterTestCase(GmpModifyFilterTestMixin, Gmpv224TestCase):
    pass



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import RequiredArgument
from gvm.protocols.gmp.requests.v224 import ReportFormatType


class GmpTriggerAlertTestMixin:
    def test_trigger_alert_without_alert_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.trigger_alert(alert_id=None, report_id="r1")

        with self.assertRaises(RequiredArgument):
            self.gmp.trigger_alert(alert_id="", report_id="r1")

    def test_trigger_alert_without_report_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.trigger_alert(alert_id="a1", report_id=None)

        with self.assertRaises(RequiredArgument):
            self.gmp.trigger_alert(alert_id="a1", report_id="")

    def test_trigger_alert(self):
        self.gmp.trigger_alert(alert_id="a1", report_id="r1")

        self.connection.send.has_been_called_with(
            b'<get_reports report_id="r1" alert_id="a1"/>'
        )

    def test_trigger_alert_with_filter_string(self):
        self.gmp.trigger_alert(
            alert_id="a1", report_id="r1", filter_string="name=foo"
        )

        self.connection.send.has_been_called_with(
            b'<get_reports report_id="r1" alert_id="a1" filter="name=foo"/>'
        )

    def test_trigger_alert_with_filter_id(self):
        self.gmp.trigger_alert(alert_id="a1", report_id="r1", filter_id="f1")

        self.connection.send.has_been_called_with(
            b'<get_reports report_id="r1" alert_id="a1" filt_id="f1"/>'
        )

    def test_trigger_alert_with_report_format_id(self):
        self.gmp.trigger_alert(
            alert_id="a1", report_id="r1", report_format_id="bar"
        )

        self.connection.send.has_been_called_with(
            b'<get_reports report_id="r1" alert_id="a1" format_id="bar"/>'
        )

    def test_trigger_alert_with_report_format_type(self):
        self.gmp.trigger_alert(
            alert_id="a1", report_id="r1", report_format_id=ReportFormatType.SVG
        )

        self.connection.send.has_been_called_with(
            b'<get_reports report_id="r1" alert_id="a1" '
            b'format_id="9e5e5deb-879e-4ecc-8be6-a71cd0875cdd"/>'
        )

    def test_trigger_alert_with_delta_report_id(self):
        self.gmp.trigger_alert(
            alert_id="a1", report_id="r1", delta_report_id="r2"
        )

        self.connection.send.has_been_called_with(
            b'<get_reports report_id="r1" alert_id="a1" delta_report_id="r2"/>'
        )



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224.entities.permissions import (
    GmpClonePermissionTestMixin,
    GmpCreatePermissionTestMixin,
    GmpDeletePermissionTestMixin,
    GmpGetPermissionsTestMixin,
    GmpGetPermissionTestMixin,
    GmpModifyPermissionTestMixin,
)
from ...gmpv226 import GMPTestCase


class GMPDeletePermissionTestCase(GmpDeletePermissionTestMixin, GMPTestCase):
    pass


class GMPGetPermissionTestCase(GmpGetPermissionTestMixin, GMPTestCase):
    pass


class GMPGetPermissionsTestCase(GmpGetPermissionsTestMixin, GMPTestCase):
    pass


class GMPClonePermissionTestCase(GmpClonePermissionTestMixin, GMPTestCase):
    pass


class GMPCreatePermissionTestCase(GmpCreatePermissionTestMixin, GMPTestCase):
    pass


class GMPModifyPermissionTestCase(GmpModifyPermissionTestMixin, GMPTestCase):
    pass



# SPDX-FileCopyrightText: 2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later


class GmpGetAgentsTestMixin:
    def test_get_agents(self):
        self.gmp.get_agents()

        self.connection.send.has_been_called_with(b"<get_agents/>")

    def test_get_agents_with_filter_string(self):
        self.gmp.get_agents(filter_string="name=agent")

        self.connection.send.has_been_called_with(
            b'<get_agents filter="name=agent"/>'
        )

    def test_get_agents_with_filter_id(self):
        self.gmp.get_agents(filter_id="f1")

        self.connection.send.has_been_called_with(b'<get_agents filt_id="f1"/>')

    def test_get_agents_with_details_true(self):
        self.gmp.get_agents(details=True)

        self.connection.send.has_been_called_with(b'<get_agents details="1"/>')

    def test_get_agents_with_details_false(self):
        self.gmp.get_agents(details=False)

        self.connection.send.has_been_called_with(b'<get_agents details="0"/>')



# SPDX-FileCopyrightText: 2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import RequiredArgument


class GmpGetCredentialStoresTestMixin:
    def test_get_credential_store(self):
        self.gmp.get_credential_store(credential_store_id="cs1")

        self.connection.send.has_been_called_with(
            b"<get_credential_stores>"
            b"<credential_store_id>cs1</credential_store_id>"
            b"</get_credential_stores>"
        )

    def test_get_credential_store_with_details(self):
        self.gmp.get_credential_store(credential_store_id="cs1", details=True)

        self.connection.send.has_been_called_with(
            b'<get_credential_stores details="1">'
            b"<credential_store_id>cs1</credential_store_id>"
            b"</get_credential_stores>"
        )

        self.gmp.get_credential_store(credential_store_id="cs1", details=False)

        self.connection.send.has_been_called_with(
            b'<get_credential_stores details="0">'
            b"<credential_store_id>cs1</credential_store_id>"
            b"</get_credential_stores>"
        )

    def test_get_credential_store_without_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.get_credential_store(credential_store_id=None)

        with self.assertRaises(RequiredArgument):
            self.gmp.get_credential_store(credential_store_id="")

    def test_get_credential_stores(self):
        self.gmp.get_credential_stores()

        self.connection.send.has_been_called_with(b"<get_credential_stores/>")

    def test_get_credential_stores_with_filter_string(self):
        self.gmp.get_credential_stores(filter_string="foo=bar")

        self.connection.send.has_been_called_with(
            b'<get_credential_stores filter="foo=bar"/>'
        )

    def test_get_credential_stores_with_filter_id(self):
        self.gmp.get_credential_stores(filter_id="f1")

        self.connection.send.has_been_called_with(
            b'<get_credential_stores filt_id="f1"/>'
        )

    def test_get_credential_stores_with_details(self):
        self.gmp.get_credential_stores(details=True)

        self.connection.send.has_been_called_with(
            b'<get_credential_stores details="1"/>'
        )

        self.gmp.get_credential_stores(details=False)

        self.connection.send.has_been_called_with(
            b'<get_credential_stores details="0"/>'
        )



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest

from gvm.errors import InvalidArgument, RequiredArgument
from gvm.protocols.gmp.requests.v224 import (
    CredentialFormat,
    Credentials,
    CredentialType,
    SnmpAuthAlgorithm,
    SnmpPrivacyAlgorithm,
)


class CredentialsTestCase(unittest.TestCase):
    def test_clone_credential_missing_credential_id(self):
        with self.assertRaises(RequiredArgument):
            Credentials.clone_credential(None)

        with self.assertRaises(RequiredArgument):
            Credentials.clone_credential("")

    def test_clone_credential(self):
        request = Credentials.clone_credential("a1")
        self.assertEqual(
            bytes(request),
            b"<create_credential><copy>a1</copy></create_credential>",
        )

    def test_create_credential_missing_name(self):
        with self.assertRaises(RequiredArgument):
            Credentials.create_credential(
                None, CredentialType.PASSWORD_ONLY, password="password"
            )

        with self.assertRaises(RequiredArgument):
            Credentials.create_credential(
                "", CredentialType.PASSWORD_ONLY, password="password"
            )

    def test_create_credential_missing_credential_type(self):
        with self.assertRaises(RequiredArgument):
            Credentials.create_credential("name", None)

        with self.assertRaises(RequiredArgument):
            Credentials.create_credential("name", "")

    def test_create_credential_invalid_credential_type(self):
        with self.assertRaises(InvalidArgument):
            Credentials.create_credential("name", "invalid")

    def test_create_credential_username_password(self):
        request = Credentials.create_credential(
            "name",
            CredentialType.USERNAME_PASSWORD,
            login="username",
            password="password",
        )
        self.assertEqual(
            bytes(request),
            b"<create_credential>"
            b"<name>name</name>"
            b"<type>up</type>"
            b"<login>username</login>"
            b"<password>password</password>"
            b"</create_credential>",
        )

    def test_create_credential_username_password_auto_generate_password(self):
        request = Credentials.create_credential(
            "name",
            CredentialType.USERNAME_PASSWORD,
            login="username",
        )
        self.assertEqual(
            bytes(request),
            b"<create_credential>"
            b"<name>name</name>"
            b"<type>up</type>"
            b"<login>username</login>"
            b"</create_credential>",
        )

    def test_create_username_password_credential_with_allow_insecure(self):
        request = Credentials.create_credential(
            "name",
            CredentialType.USERNAME_PASSWORD,
            login="username",
            password="password",
            allow_insecure=True,
        )
        self.assertEqual(
            bytes(request),
            b"<create_credential>"
            b"<name>name</name>"
            b"<type>up</type>"
            b"<allow_insecure>1</allow_insecure>"
            b"<login>username</login>"
            b"<password>password</password>"
            b"</create_credential>",
        )

    def test_create_credential_username_password_with_comment(self):
        request = Credentials.create_credential(
            "name",
            CredentialType.USERNAME_PASSWORD,
            login="username",
            password="password",
            comment="comment",
        )
        self.assertEqual(
            bytes(request),
            b"<create_credential>"
            b"<name>name</name>"
            b"<type>up</type>"
            b"<comment>comment</comment>"
            b"<login>username</login>"
            b"<password>password</password>"
            b"</create_credential>",
        )

    def test_create_credential_username_password_missing_login(self):
        with self.assertRaises(RequiredArgument):
            Credentials.create_credential(
                "name", CredentialType.USERNAME_PASSWORD, password="password"
            )

        with self.assertRaises(RequiredArgument):
            Credentials.create_credential(
                "name",
                CredentialType.USERNAME_PASSWORD,
                login="",
                password="password",
            )

    def test_create_credential_client_certificate(self):
        request = Credentials.create_credential(
            "name",
            CredentialType.CLIENT_CERTIFICATE,
            certificate="certificate",
        )
        self.assertEqual(
            bytes(request),
            b"<create_credential>"
            b"<name>name</name>"
            b"<type>cc</type>"
            b"<certificate>certificate</certificate>"
            b"</create_credential>",
        )

    def test_create_credential_client_certificate_with_private_key(self):
        request = Credentials.create_credential(
            "name",
            CredentialType.CLIENT_CERTIFICATE,
            certificate="certificate",
            private_key="private_key",
        )
        self.assertEqual(
            bytes(request),
            b"<create_credential>"
            b"<name>name</name>"
            b"<type>cc</type>"
            b"<certificate>certificate</certificate>"
            b"<key>"
            b"<private>private_key</private>"
            b"</key>"
            b"</create_credential>",
        )

    def test_create_credential_client_certificate_missing_certificate(self):
        with self.assertRaises(RequiredArgument):
            Credentials.create_credential(
                "name", CredentialType.CLIENT_CERTIFICATE
            )

        with self.assertRaises(RequiredArgument):
            Credentials.create_credential(
                "name", CredentialType.CLIENT_CERTIFICATE, certificate=""
            )

    def test_create_credential_username_ssh_key(self):
        request = Credentials.create_credential(
            "name",
            CredentialType.USERNAME_SSH_KEY,
            login="username",
            private_key="private",
        )

        self.assertEqual(
            bytes(request),
            b"<create_credential>"
            b"<name>name</name>"
            b"<type>usk</type>"
            b"<login>username</login>"
            b"<key>"
            b"<private>private</private>"
            b"</key>"
            b"</create_credential>",
        )

    def test_create_credential_username_ssh_key_with_passphrase(self):
        request = Credentials.create_credential(
            "name",
            CredentialType.USERNAME_SSH_KEY,
            login="username",
            private_key="private",
            key_phrase="passphrase",
        )

        self.assertEqual(
            bytes(request),
            b"<create_credential>"
            b"<name>name</name>"
            b"<type>usk</type>"
            b"<login>username</login>"
            b"<key>"
            b"<private>private</private>"
            b"<phrase>passphrase</phrase>"
            b"</key>"
            b"</create_credential>",
        )

    def test_create_credential_username_ssh_key_auto_generate_key(self):
        request = Credentials.create_credential(
            "name",
            CredentialType.USERNAME_SSH_KEY,
            login="username",
        )

        self.assertEqual(
            bytes(request),
            b"<create_credential>"
            b"<name>name</name>"
            b"<type>usk</type>"
            b"<login>username</login>"
            b"</create_credential>",
        )

    def test_create_credential_username_ssh_key_missing_private_key(self):
        with self.assertRaises(RequiredArgument):
            Credentials.create_credential(
                "name",
                CredentialType.USERNAME_SSH_KEY,
                login="username",
                private_key="",
            )

    def test_create_credential_username_ssh_key_missing_login(self):
        with self.assertRaises(RequiredArgument):
            Credentials.create_credential(
                "name", CredentialType.USERNAME_SSH_KEY, private_key="private"
            )

        with self.assertRaises(RequiredArgument):
            Credentials.create_credential(
                "name",
                CredentialType.USERNAME_SSH_KEY,
                login="",
                private_key="private",
            )

    def test_create_credential_snmp_with_auth_algorithm_md5(self):
        request = Credentials.create_credential(
            "name",
            CredentialType.SNMP,
            auth_algorithm=SnmpAuthAlgorithm.MD5,
            login="username",
        )

        self.assertEqual(
            bytes(request),
            b"<create_credential>"
            b"<name>name</name>"
            b"<type>snmp</type>"
            b"<login>username</login>"
            b"<auth_algorithm>md5</auth_algorithm>"
            b"</create_credential>",
        )

    def test_create_credential_snmp_with_auth_algorithm_sha1(self):
        request = Credentials.create_credential(
            "name",
            CredentialType.SNMP,
            auth_algorithm=SnmpAuthAlgorithm.SHA1,
            login="username",
        )

        self.assertEqual(
            bytes(request),
            b"<create_credential>"
            b"<name>name</name>"
            b"<type>snmp</type>"
            b"<login>username</login>"
            b"<auth_algorithm>sha1</auth_algorithm>"
            b"</create_credential>",
        )

    def test_create_credential_snmp_missing_auth_algorithm(self):
        with self.assertRaises(RequiredArgument):
            Credentials.create_credential(
                "name", CredentialType.SNMP, login="username"
            )
        with self.assertRaises(RequiredArgument):
            Credentials.create_credential(
                "name",
                CredentialType.SNMP,
                login="username",
                auth_algorithm=None,
            )
        with self.assertRaises(RequiredArgument):
            Credentials.create_credential(
                "name",
                CredentialType.SNMP,
                login="username",
                auth_algorithm="",
            )

    def test_create_credential_snmp_missing_login(self):
        with self.assertRaises(RequiredArgument):
            Credentials.create_credential(
                "name",
                CredentialType.SNMP,
                auth_algorithm=SnmpAuthAlgorithm.MD5,
            )

        with self.assertRaises(RequiredArgument):
            Credentials.create_credential(
                "name",
                CredentialType.SNMP,
                auth_algorithm=SnmpAuthAlgorithm.MD5,
                login="",
            )

    def test_create_credential_snmp_with_community(self):
        request = Credentials.create_credential(
            "name",
            CredentialType.SNMP,
            auth_algorithm=SnmpAuthAlgorithm.MD5,
            login="username",
            community="community",
        )

        self.assertEqual(
            bytes(request),
            b"<create_credential>"
            b"<name>name</name>"
            b"<type>snmp</type>"
            b"<login>username</login>"
            b"<auth_algorithm>md5</auth_algorithm>"
            b"<community>community</community>"
            b"</create_credential>",
        )

    def test_create_credential_snmp_with_password(self):
        request = Credentials.create_credential(
            "name",
            CredentialType.SNMP,
            auth_algorithm=SnmpAuthAlgorithm.MD5,
            login="username",
            password="password",
        )

        self.assertEqual(
            bytes(request),
            b"<create_credential>"
            b"<name>name</name>"
            b"<type>snmp</type>"
            b"<login>username</login>"
            b"<password>password</password>"
            b"<auth_algorithm>md5</auth_algorithm>"
            b"</create_credential>",
        )

    def test_create_credential_snmp_with_privacy_algorithm(self):
        request = Credentials.create_credential(
            "name",
            CredentialType.SNMP,
            auth_algorithm=SnmpAuthAlgorithm.MD5,
            login="username",
            privacy_algorithm=SnmpPrivacyAlgorithm.DES,
        )

        self.assertEqual(
            bytes(request),
            b"<create_credential>"
            b"<name>name</name>"
            b"<type>snmp</type>"
            b"<login>username</login>"
            b"<auth_algorithm>md5</auth_algorithm>"
            b"<privacy>"
            b"<algorithm>des</algorithm>"
            b"</privacy>"
            b"</create_credential>",
        )

    def test_create_credential_snmp_with_privacy_algorithm_and_privacy_password(
        self,
    ):
        request = Credentials.create_credential(
            "name",
            CredentialType.SNMP,
            auth_algorithm=SnmpAuthAlgorithm.MD5,
            login="username",
            privacy_algorithm=SnmpPrivacyAlgorithm.DES,
            privacy_password="password",
        )

        self.assertEqual(
            bytes(request),
            b"<create_credential>"
            b"<name>name</name>"
            b"<type>snmp</type>"
            b"<login>username</login>"
            b"<auth_algorithm>md5</auth_algorithm>"
            b"<privacy>"
            b"<algorithm>des</algorithm>"
            b"<password>password</password>"
            b"</privacy>"
            b"</create_credential>",
        )

    def test_create_credential_pgp(self):
        request = Credentials.create_credential(
            "name",
            CredentialType.PGP_ENCRYPTION_KEY,
            public_key="public_key",
        )

        self.assertEqual(
            bytes(request),
            b"<create_credential>"
            b"<name>name</name>"
            b"<type>pgp</type>"
            b"<key>"
            b"<public>public_key</public>"
            b"</key>"
            b"</create_credential>",
        )

    def test_create_credential_pgp_missing_key(self):
        with self.assertRaises(RequiredArgument):
            Credentials.create_credential(
                "name", CredentialType.PGP_ENCRYPTION_KEY
            )

        with self.assertRaises(RequiredArgument):
            Credentials.create_credential(
                "name", CredentialType.PGP_ENCRYPTION_KEY, public_key=None
            )

        with self.assertRaises(RequiredArgument):
            Credentials.create_credential(
                "name", CredentialType.PGP_ENCRYPTION_KEY, public_key=""
            )

    def test_create_credential_password_only(self):
        request = Credentials.create_credential(
            "name",
            CredentialType.PASSWORD_ONLY,
            password="password",
        )
        self.assertEqual(
            bytes(request),
            b"<create_credential>"
            b"<name>name</name>"
            b"<type>pw</type>"
            b"<password>password</password>"
            b"</create_credential>",
        )

    def test_create_credential_password_only_missing_password(self):
        with self.assertRaises(RequiredArgument):
            Credentials.create_credential("name", CredentialType.PASSWORD_ONLY)

        with self.assertRaises(RequiredArgument):
            Credentials.create_credential(
                "name", CredentialType.PASSWORD_ONLY, password=None
            )

        with self.assertRaises(RequiredArgument):
            Credentials.create_credential(
                "name", CredentialType.PASSWORD_ONLY, password=""
            )

    def test_create_credential_smime(self):
        request = Credentials.create_credential(
            "name",
            CredentialType.SMIME_CERTIFICATE,
            certificate="certificate",
        )
        self.assertEqual(
            bytes(request),
            b"<create_credential>"
            b"<name>name</name>"
            b"<type>smime</type>"
            b"<certificate>certificate</certificate>"
            b"</create_credential>",
        )

    def test_create_credential_smime_missing_certificate(self):
        with self.assertRaises(RequiredArgument):
            Credentials.create_credential(
                "name",
                CredentialType.SMIME_CERTIFICATE,
            )

        with self.assertRaises(RequiredArgument):
            Credentials.create_credential(
                "name",
                CredentialType.SMIME_CERTIFICATE,
                certificate=None,
            )

        with self.assertRaises(RequiredArgument):
            Credentials.create_credential(
                "name",
                CredentialType.SMIME_CERTIFICATE,
                certificate="",
            )

    def test_delete_alert(self):
        request = Credentials.delete_credential("a1")
        self.assertEqual(
            bytes(request),
            b'<delete_credential credential_id="a1" ultimate="0"/>',
        )

    def test_delete_alert_with_ultimate(self):
        request = Credentials.delete_credential("a1", ultimate=False)
        self.assertEqual(
            bytes(request),
            b'<delete_credential credential_id="a1" ultimate="0"/>',
        )

        request = Credentials.delete_credential("a1", ultimate=True)
        self.assertEqual(
            bytes(request),
            b'<delete_credential credential_id="a1" ultimate="1"/>',
        )

    def test_delete_alert_missing_credential_id(self):
        with self.assertRaises(RequiredArgument):
            Credentials.delete_credential(None)

        with self.assertRaises(RequiredArgument):
            Credentials.delete_credential("")

    def test_get_credentials(self):
        request = Credentials.get_credentials()
        self.assertEqual(
            bytes(request),
            b"<get_credentials/>",
        )

    def test_get_credentials_with_filter_string(self):
        request = Credentials.get_credentials(filter_string="name")
        self.assertEqual(
            bytes(request),
            b'<get_credentials filter="name"/>',
        )

    def test_get_credentials_with_scanners(self):
        request = Credentials.get_credentials(scanners=True)
        self.assertEqual(
            bytes(request),
            b'<get_credentials scanners="1"/>',
        )

    def test_get_credentials_with_trash(self):
        request = Credentials.get_credentials(trash=True)
        self.assertEqual(
            bytes(request),
            b'<get_credentials trash="1"/>',
        )

    def test_get_credentials_with_targets(self):
        request = Credentials.get_credentials(targets=True)
        self.assertEqual(
            bytes(request),
            b'<get_credentials targets="1"/>',
        )

    def test_get_credential(self):
        request = Credentials.get_credential("a1")
        self.assertEqual(
            bytes(request),
            b'<get_credentials credential_id="a1"/>',
        )

    def test_get_credential_with_scanners(self):
        request = Credentials.get_credential("a1", scanners=True)
        self.assertEqual(
            bytes(request),
            b'<get_credentials credential_id="a1" scanners="1"/>',
        )

    def test_get_credential_with_targets(self):
        request = Credentials.get_credential("a1", targets=True)
        self.assertEqual(
            bytes(request),
            b'<get_credentials credential_id="a1" targets="1"/>',
        )

    def test_get_credential_with_credential_format(self):
        request = Credentials.get_credential("a1", credential_format="key")
        self.assertEqual(
            bytes(request),
            b'<get_credentials credential_id="a1" format="key"/>',
        )

        request = Credentials.get_credential(
            "a1", credential_format=CredentialFormat.KEY
        )
        self.assertEqual(
            bytes(request),
            b'<get_credentials credential_id="a1" format="key"/>',
        )

        request = Credentials.get_credential(
            "a1", credential_format=CredentialFormat.EXE
        )
        self.assertEqual(
            bytes(request),
            b'<get_credentials credential_id="a1" format="exe"/>',
        )

        request = Credentials.get_credential(
            "a1", credential_format=CredentialFormat.DEB
        )
        self.assertEqual(
            bytes(request),
            b'<get_credentials credential_id="a1" format="deb"/>',
        )

        request = Credentials.get_credential(
            "a1", credential_format=CredentialFormat.PEM
        )
        self.assertEqual(
            bytes(request),
            b'<get_credentials credential_id="a1" format="pem"/>',
        )

        request = Credentials.get_credential(
            "a1", credential_format=CredentialFormat.RPM
        )
        self.assertEqual(
            bytes(request),
            b'<get_credentials credential_id="a1" format="rpm"/>',
        )

    def test_get_credential_missing_credential_id(self):
        with self.assertRaises(RequiredArgument):
            Credentials.get_credential(None)

        with self.assertRaises(RequiredArgument):
            Credentials.get_credential("")

    def test_modify_credential_with_name(self):
        request = Credentials.modify_credential("a1", name="name")
        self.assertEqual(
            bytes(request),
            b'<modify_credential credential_id="a1">'
            b"<name>name</name>"
            b"</modify_credential>",
        )

    def test_modify_credential_with_comment(self):
        request = Credentials.modify_credential("a1", comment="comment")
        self.assertEqual(
            bytes(request),
            b'<modify_credential credential_id="a1">'
            b"<comment>comment</comment>"
            b"</modify_credential>",
        )

    def test_modify_credential_with_allow_insecure(self):
        request = Credentials.modify_credential("a1", allow_insecure=True)
        self.assertEqual(
            bytes(request),
            b'<modify_credential credential_id="a1">'
            b"<allow_insecure>1</allow_insecure>"
            b"</modify_credential>",
        )
        request = Credentials.modify_credential("a1", allow_insecure=False)
        self.assertEqual(
            bytes(request),
            b'<modify_credential credential_id="a1">'
            b"<allow_insecure>0</allow_insecure>"
            b"</modify_credential>",
        )

    def test_modify_credential_with_certificate(self):
        request = Credentials.modify_credential("a1", certificate="certificate")
        self.assertEqual(
            bytes(request),
            b'<modify_credential credential_id="a1">'
            b"<certificate>certificate</certificate>"
            b"</modify_credential>",
        )

    def test_modify_credential_with_key_phrase_and_private_key(self):
        request = Credentials.modify_credential(
            "a1", key_phrase="passphrase", private_key="private"
        )
        self.assertEqual(
            bytes(request),
            b'<modify_credential credential_id="a1">'
            b"<key>"
            b"<phrase>passphrase</phrase>"
            b"<private>private</private>"
            b"</key>"
            b"</modify_credential>",
        )

    def test_modify_credential_missing_key_phrase_or_private_key(self):
        with self.assertRaises(RequiredArgument):
            Credentials.modify_credential("a1", key_phrase="passphrase")

        with self.assertRaises(RequiredArgument):
            Credentials.modify_credential("a1", private_key="private")

    def test_modify_credential_with_login(self):
        request = Credentials.modify_credential("a1", login="username")
        self.assertEqual(
            bytes(request),
            b'<modify_credential credential_id="a1">'
            b"<login>username</login>"
            b"</modify_credential>",
        )

    def test_modify_credential_with_password(self):
        request = Credentials.modify_credential("a1", password="password")
        self.assertEqual(
            bytes(request),
            b'<modify_credential credential_id="a1">'
            b"<password>password</password>"
            b"</modify_credential>",
        )

    def test_modify_credential_with_auth_algorithm(self):
        request = Credentials.modify_credential(
            "a1", auth_algorithm=SnmpAuthAlgorithm.MD5
        )
        self.assertEqual(
            bytes(request),
            b'<modify_credential credential_id="a1">'
            b"<auth_algorithm>md5</auth_algorithm>"
            b"</modify_credential>",
        )

        request = Credentials.modify_credential("a1", auth_algorithm="md5")
        self.assertEqual(
            bytes(request),
            b'<modify_credential credential_id="a1">'
            b"<auth_algorithm>md5</auth_algorithm>"
            b"</modify_credential>",
        )

        request = Credentials.modify_credential(
            "a1", auth_algorithm=SnmpAuthAlgorithm.SHA1
        )
        self.assertEqual(
            bytes(request),
            b'<modify_credential credential_id="a1">'
            b"<auth_algorithm>sha1</auth_algorithm>"
            b"</modify_credential>",
        )

    def test_modify_credential_with_community(self):
        request = Credentials.modify_credential("a1", community="community")
        self.assertEqual(
            bytes(request),
            b'<modify_credential credential_id="a1">'
            b"<community>community</community>"
            b"</modify_credential>",
        )

    def test_modify_credential_with_privacy_algorithm(self):
        request = Credentials.modify_credential(
            "a1", privacy_algorithm=SnmpPrivacyAlgorithm.DES
        )
        self.assertEqual(
            bytes(request),
            b'<modify_credential credential_id="a1">'
            b"<privacy>"
            b"<algorithm>des</algorithm>"
            b"</privacy>"
            b"</modify_credential>",
        )

        request = Credentials.modify_credential("a1", privacy_algorithm="des")
        self.assertEqual(
            bytes(request),
            b'<modify_credential credential_id="a1">'
            b"<privacy>"
            b"<algorithm>des</algorithm>"
            b"</privacy>"
            b"</modify_credential>",
        )

        request = Credentials.modify_credential(
            "a1", privacy_algorithm=SnmpPrivacyAlgorithm.AES
        )
        self.assertEqual(
            bytes(request),
            b'<modify_credential credential_id="a1">'
            b"<privacy>"
            b"<algorithm>aes</algorithm>"
            b"</privacy>"
            b"</modify_credential>",
        )

    def test_modify_credential_with_privacy_password(self):
        request = Credentials.modify_credential(
            "a1", privacy_password="password"
        )
        self.assertEqual(
            bytes(request),
            b'<modify_credential credential_id="a1">'
            b"<privacy>"
            b"<password>password</password>"
            b"</privacy>"
            b"</modify_credential>",
        )

    def test_modify_credential_with_public_key(self):
        request = Credentials.modify_credential("a1", public_key="public_key")
        self.assertEqual(
            bytes(request),
            b'<modify_credential credential_id="a1">'
            b"<key>"
            b"<public>public_key</public>"
            b"</key>"
            b"</modify_credential>",
        )

    def test_modify_credential_missing_credential_id(self):
        with self.assertRaises(RequiredArgument):
            Credentials.modify_credential(None)

        with self.assertRaises(RequiredArgument):
            Credentials.modify_credential("")



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from .test_clone_audit import GmpCloneAuditTestMixin
from .test_create_audit import GmpCreateAuditTestMixin
from .test_delete_audit import GmpDeleteAuditTestMixin
from .test_get_audit import GmpGetAuditTestMixin
from .test_get_audits import GmpGetAuditsTestMixin
from .test_modify_audit import GmpModifyAuditTestMixin
from .test_resume_audit import GmpResumeAuditTestMixin
from .test_start_audit import GmpStartAuditTestMixin
from .test_stop_audit import GmpStopAuditTestMixin

__all__ = (
    "GmpCloneAuditTestMixin",
    "GmpCreateAuditTestMixin",
    "GmpDeleteAuditTestMixin",
    "GmpGetAuditTestMixin",
    "GmpGetAuditsTestMixin",
    "GmpModifyAuditTestMixin",
    "GmpResumeAuditTestMixin",
    "GmpStartAuditTestMixin",
    "GmpStopAuditTestMixin",
)



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import RequiredArgument
from gvm.protocols.gmp.requests.v224 import ReportFormatType


class GmpModifyReportFormatTestMixin:
    def test_modify_report_format(self):
        self.gmp.modify_report_format(report_format_id="rf1")

        self.connection.send.has_been_called_with(
            b'<modify_report_format report_format_id="rf1"/>'
        )

    def test_modify_report_format_missing_report_format_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_report_format(report_format_id=None)

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_report_format(report_format_id="")

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_report_format("")

    def test_modify_report_format_with_summary(self):
        self.gmp.modify_report_format(report_format_id="rf1", summary="foo")

        self.connection.send.has_been_called_with(
            b'<modify_report_format report_format_id="rf1">'
            b"<summary>foo</summary>"
            b"</modify_report_format>"
        )

    def test_modify_report_format_with_name(self):
        self.gmp.modify_report_format(report_format_id="rf1", name="foo")

        self.connection.send.has_been_called_with(
            b'<modify_report_format report_format_id="rf1">'
            b"<name>foo</name>"
            b"</modify_report_format>"
        )

    def test_modify_report_format_with_name_and_type(self):
        self.gmp.modify_report_format(
            report_format_id=ReportFormatType.XML, name="foo"
        )

        report_format_id = ReportFormatType.from_string("xml").value
        self.connection.send.has_been_called_with(
            f'<modify_report_format report_format_id="{report_format_id}">'
            "<name>foo</name></modify_report_format>".encode()
        )

    def test_modify_report_format_with_active(self):
        self.gmp.modify_report_format(report_format_id="rf1", active=True)

        self.connection.send.has_been_called_with(
            b'<modify_report_format report_format_id="rf1">'
            b"<active>1</active>"
            b"</modify_report_format>"
        )

        self.gmp.modify_report_format(report_format_id="rf1", active=False)

        self.connection.send.has_been_called_with(
            b'<modify_report_format report_format_id="rf1">'
            b"<active>0</active>"
            b"</modify_report_format>"
        )

    def test_modify_report_format_with_param_name(self):
        self.gmp.modify_report_format(report_format_id="rf1", param_name="foo")

        self.connection.send.has_been_called_with(
            b'<modify_report_format report_format_id="rf1">'
            b"<param>"
            b"<name>foo</name>"
            b"</param>"
            b"</modify_report_format>"
        )

    def test_modify_report_format_with_param_name_and_value(self):
        self.gmp.modify_report_format(
            report_format_id="rf1", param_name="foo", param_value="bar"
        )

        self.connection.send.has_been_called_with(
            b'<modify_report_format report_format_id="rf1">'
            b"<param>"
            b"<name>foo</name>"
            b"<value>bar</value>"
            b"</param>"
            b"</modify_report_format>"
        )

        self.gmp.modify_report_format(
            report_format_id="rf1", param_name="foo", param_value=""
        )

        self.connection.send.has_been_called_with(
            b'<modify_report_format report_format_id="rf1">'
            b"<param>"
            b"<name>foo</name>"
            b"<value></value>"
            b"</param>"
            b"</modify_report_format>"
        )



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224.entities.vulnerabilities import (
    GmpGetVulnerabilitiesTestMixin,
    GmpGetVulnerabilityTestMixin,
)
from ...gmpv226 import GMPTestCase


class GMPGetVulnerabilityTestCase(GmpGetVulnerabilityTestMixin, GMPTestCase):
    pass


class GMPGetVulnerabilitiesTestCase(
    GmpGetVulnerabilitiesTestMixin, GMPTestCase
):
    pass



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from . import GMPTestCase


class GmpWithStatementTestMixin:
    def test_with_statement(self):
        self.connection.connect.has_not_been_called()
        self.connection.disconnect.has_not_been_called()

        with self.gmp:
            pass

        self.connection.connect.has_been_called()
        self.connection.disconnect.has_been_called()


class Gmpv225WithStatementTestCase(GmpWithStatementTestMixin, GMPTestCase):
    pass



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from .test_authenticate import GmpAuthenticateTestMixin
from .test_describe_auth import GmpDescribeAuthTestMixin
from .test_modify_auth import GmpModifyAuthTestMixin

__all__ = (
    "GmpAuthenticateTestMixin",
    "GmpDescribeAuthTestMixin",
    "GmpModifyAuthTestMixin",
)



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import RequiredArgument


class GmpGetOverrideTestMixin:
    def test_get_override(self):
        self.gmp.get_override("o1")

        self.connection.send.has_been_called_with(
            b'<get_overrides override_id="o1" details="1"/>'
        )

        self.gmp.get_override(override_id="o1")

        self.connection.send.has_been_called_with(
            b'<get_overrides override_id="o1" details="1"/>'
        )

    def test_get_override_missing_override_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.get_override(override_id=None)

        with self.assertRaises(RequiredArgument):
            self.gmp.get_override("")



# SPDX-FileCopyrightText: 2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

from gvm.errors import RequiredArgument


class GmpModifyAgentsTestMixin:
    def test_modify_agents_basic(self):
        self.gmp.modify_agents(agent_ids=["agent-123"])

        self.connection.send.has_been_called_with(
            b"<modify_agent>"
            b'<agents><agent id="agent-123"/></agents>'
            b"</modify_agent>"
        )

    def test_modify_agents_with_authorized_only(self):
        self.gmp.modify_agents(
            agent_ids=["agent-123", "agent-456"], authorized=True
        )

        self.connection.send.has_been_called_with(
            b"<modify_agent>"
            b'<agents><agent id="agent-123"/><agent id="agent-456"/></agents>'
            b"<authorized>1</authorized>"
            b"</modify_agent>"
        )

    def test_modify_agents_with_update_to_latest_only(self):
        self.gmp.modify_agents(
            agent_ids=["agent-123", "agent-456"], update_to_latest=True
        )

        self.connection.send.has_been_called_with(
            b"<modify_agent>"
            b'<agents><agent id="agent-123"/><agent id="agent-456"/></agents>'
            b"<update_to_latest>1</update_to_latest>"
            b"</modify_agent>"
        )

    def test_modify_agents_with_full_config_and_comment(self):
        cfg = {
            "agent_control": {
                "retry": {
                    "attempts": 6,
                    "delay_in_seconds": 60,
                    "max_jitter_in_seconds": 10,
                }
            },
            "agent_script_executor": {
                "bulk_size": 2,
                "bulk_throttle_time_in_ms": 300,
                "indexer_dir_depth": 100,
                "scheduler_cron_time": ["0 */12 * * *"],
            },
            "heartbeat": {"interval_in_seconds": 300, "miss_until_inactive": 1},
        }

        self.gmp.modify_agents(
            agent_ids=["agent-123", "agent-456"],
            authorized=True,
            update_to_latest=True,
            config=cfg,
            comment="Updated agents",
        )

        self.connection.send.has_been_called_with(
            b"<modify_agent>"
            b'<agents><agent id="agent-123"/><agent id="agent-456"/></agents>'
            b"<authorized>1</authorized>"
            b"<update_to_latest>1</update_to_latest>"
            b"<config>"
            b"<agent_control>"
            b"<retry>"
            b"<attempts>6</attempts>"
            b"<delay_in_seconds>60</delay_in_seconds>"
            b"<max_jitter_in_seconds>10</max_jitter_in_seconds>"
            b"</retry>"
            b"</agent_control>"
            b"<agent_script_executor>"
            b"<bulk_size>2</bulk_size>"
            b"<bulk_throttle_time_in_ms>300</bulk_throttle_time_in_ms>"
            b"<indexer_dir_depth>100</indexer_dir_depth>"
            b'<scheduler_cron_time is_list="1">'
            b"<item>0 */12 * * *</item>"
            b"</scheduler_cron_time>"
            b"</agent_script_executor>"
            b"<heartbeat>"
            b"<interval_in_seconds>300</interval_in_seconds>"
            b"<miss_until_inactive>1</miss_until_inactive>"
            b"</heartbeat>"
            b"</config>"
            b"<comment>Updated agents</comment>"
            b"</modify_agent>"
        )

    def test_modify_agents_omits_optional_elements_when_none(self):
        cfg = {
            "agent_control": {
                "retry": {
                    "attempts": 6,
                    "delay_in_seconds": 60,
                    "max_jitter_in_seconds": 10,
                }
            },
            "agent_script_executor": {
                "bulk_size": 2,
                "bulk_throttle_time_in_ms": 300,
                "indexer_dir_depth": 100,
                "scheduler_cron_time": ["0 */12 * * *"],
            },
            "heartbeat": {
                "interval_in_seconds": 300,
                "miss_until_inactive": 1,
            },
        }

        self.gmp.modify_agents(
            agent_ids=["agent-123"],
            authorized=None,
            update_to_latest=None,
            config=cfg,
            comment=None,
        )

        self.connection.send.has_been_called_with(
            b"<modify_agent>"
            b'<agents><agent id="agent-123"/></agents>'
            b"<config>"
            b"<agent_control>"
            b"<retry>"
            b"<attempts>6</attempts>"
            b"<delay_in_seconds>60</delay_in_seconds>"
            b"<max_jitter_in_seconds>10</max_jitter_in_seconds>"
            b"</retry>"
            b"</agent_control>"
            b"<agent_script_executor>"
            b"<bulk_size>2</bulk_size>"
            b"<bulk_throttle_time_in_ms>300</bulk_throttle_time_in_ms>"
            b"<indexer_dir_depth>100</indexer_dir_depth>"
            b'<scheduler_cron_time is_list="1">'
            b"<item>0 */12 * * *</item>"
            b"</scheduler_cron_time>"
            b"</agent_script_executor>"
            b"<heartbeat>"
            b"<interval_in_seconds>300</interval_in_seconds>"
            b"<miss_until_inactive>1</miss_until_inactive>"
            b"</heartbeat>"
            b"</config>"
            b"</modify_agent>"
        )

    def test_modify_agents_with_full_config_with_missing_element(self):
        cfg = {
            "agent_control": {
                "retry": {
                    "attempts": 6,
                    "delay_in_seconds": 60,
                    "max_jitter_in_seconds": 10,
                }
            },
            "agent_script_executor": {
                "bulk_size": 2,
                "bulk_throttle_time_in_ms": 300,
                "indexer_dir_depth": 100,
                # scheduler_cron_time is missing
            },
            "heartbeat": {"interval_in_seconds": 300, "miss_until_inactive": 1},
        }

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_agents(
                agent_ids=["agent-123", "agent-456"],
                authorized=True,
                config=cfg,
                comment="Updated agents",
            )

    def test_modify_agents_without_ids_raises(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_agents(agent_ids=[])

    def test_modify_agents_scheduler_empty_list_raises(self):
        cfg = {
            "agent_control": {
                "retry": {
                    "attempts": 6,
                    "delay_in_seconds": 60,
                    "max_jitter_in_seconds": 10,
                }
            },
            "agent_script_executor": {
                "bulk_size": 2,
                "bulk_throttle_time_in_ms": 300,
                "indexer_dir_depth": 100,
                "scheduler_cron_time": [],
            },
            "heartbeat": {"interval_in_seconds": 300, "miss_until_inactive": 1},
        }
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_agents(agent_ids=["agent-123"], config=cfg)

    def test_modify_agents_scheduler_with_empty_item_raises(self):
        cfg = {
            "agent_control": {
                "retry": {
                    "attempts": 6,
                    "delay_in_seconds": 60,
                    "max_jitter_in_seconds": 10,
                }
            },
            "agent_script_executor": {
                "bulk_size": 2,
                "bulk_throttle_time_in_ms": 300,
                "indexer_dir_depth": 100,
                "scheduler_cron_time": ["", "   "],
            },
            "heartbeat": {"interval_in_seconds": 300, "miss_until_inactive": 1},
        }
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_agents(agent_ids=["agent-123"], config=cfg)

    def test_modify_agents_missing_agent_control_raises(self):
        cfg = {
            # "agent_control": missing
            "agent_script_executor": {
                "bulk_size": 2,
                "bulk_throttle_time_in_ms": 300,
                "indexer_dir_depth": 100,
                "scheduler_cron_time": ["0 */12 * * *"],
            },
            "heartbeat": {"interval_in_seconds": 300, "miss_until_inactive": 1},
        }
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_agents(agent_ids=["agent-123"], config=cfg)

    def test_modify_agents_missing_retry_block_raises(self):
        cfg = {
            "agent_control": {  # retry missing
                # "retry": {}
            },
            "agent_script_executor": {
                "bulk_size": 2,
                "bulk_throttle_time_in_ms": 300,
                "indexer_dir_depth": 100,
                "scheduler_cron_time": ["0 */12 * * *"],
            },
            "heartbeat": {"interval_in_seconds": 300, "miss_until_inactive": 1},
        }
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_agents(agent_ids=["agent-123"], config=cfg)

    def test_modify_agents_missing_retry_attempts_raises(self):
        cfg = {
            "agent_control": {
                "retry": {
                    # "attempts": 6,
                    "delay_in_seconds": 60,
                    "max_jitter_in_seconds": 10,
                }
            },
            "agent_script_executor": {
                "bulk_size": 2,
                "bulk_throttle_time_in_ms": 300,
                "indexer_dir_depth": 100,
                "scheduler_cron_time": ["0 */12 * * *"],
            },
            "heartbeat": {"interval_in_seconds": 300, "miss_until_inactive": 1},
        }
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_agents(agent_ids=["agent-123"], config=cfg)

    def test_modify_agents_missing_retry_delay_raises(self):
        cfg = {
            "agent_control": {
                "retry": {
                    "attempts": 6,
                    # "delay_in_seconds": 60,
                    "max_jitter_in_seconds": 10,
                }
            },
            "agent_script_executor": {
                "bulk_size": 2,
                "bulk_throttle_time_in_ms": 300,
                "indexer_dir_depth": 100,
                "scheduler_cron_time": ["0 */12 * * *"],
            },
            "heartbeat": {"interval_in_seconds": 300, "miss_until_inactive": 1},
        }
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_agents(agent_ids=["agent-123"], config=cfg)

    def test_modify_agents_missing_retry_max_jitter_raises(self):
        cfg = {
            "agent_control": {
                "retry": {
                    "attempts": 6,
                    "delay_in_seconds": 60,
                    # "max_jitter_in_seconds": 10,
                }
            },
            "agent_script_executor": {
                "bulk_size": 2,
                "bulk_throttle_time_in_ms": 300,
                "indexer_dir_depth": 100,
                "scheduler_cron_time": ["0 */12 * * *"],
            },
            "heartbeat": {"interval_in_seconds": 300, "miss_until_inactive": 1},
        }
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_agents(agent_ids=["agent-123"], config=cfg)

    def test_modify_agents_missing_agent_script_executor_raises(self):
        cfg = {
            "agent_control": {
                "retry": {
                    "attempts": 6,
                    "delay_in_seconds": 60,
                    "max_jitter_in_seconds": 10,
                }
            },
            # "agent_script_executor": missing
            "heartbeat": {"interval_in_seconds": 300, "miss_until_inactive": 1},
        }
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_agents(agent_ids=["agent-123"], config=cfg)

    def test_modify_agents_missing_bulk_size_raises(self):
        cfg = {
            "agent_control": {
                "retry": {
                    "attempts": 6,
                    "delay_in_seconds": 60,
                    "max_jitter_in_seconds": 10,
                }
            },
            "agent_script_executor": {
                # "bulk_size": 2,
                "bulk_throttle_time_in_ms": 300,
                "indexer_dir_depth": 100,
                "scheduler_cron_time": ["0 */12 * * *"],
            },
            "heartbeat": {"interval_in_seconds": 300, "miss_until_inactive": 1},
        }
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_agents(agent_ids=["agent-123"], config=cfg)

    def test_modify_agents_missing_bulk_throttle_time_in_ms_raises(self):
        cfg = {
            "agent_control": {
                "retry": {
                    "attempts": 6,
                    "delay_in_seconds": 60,
                    "max_jitter_in_seconds": 10,
                }
            },
            "agent_script_executor": {
                "bulk_size": 2,
                # "bulk_throttle_time_in_ms": 300,
                "indexer_dir_depth": 100,
                "scheduler_cron_time": ["0 */12 * * *"],
            },
            "heartbeat": {"interval_in_seconds": 300, "miss_until_inactive": 1},
        }
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_agents(agent_ids=["agent-123"], config=cfg)

    def test_modify_agents_missing_indexer_dir_depth_raises(self):
        cfg = {
            "agent_control": {
                "retry": {
                    "attempts": 6,
                    "delay_in_seconds": 60,
                    "max_jitter_in_seconds": 10,
                }
            },
            "agent_script_executor": {
                "bulk_size": 2,
                "bulk_throttle_time_in_ms": 300,
                # "indexer_dir_depth": 100,
                "scheduler_cron_time": ["0 */12 * * *"],
            },
            "heartbeat": {"interval_in_seconds": 300, "miss_until_inactive": 1},
        }
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_agents(agent_ids=["agent-123"], config=cfg)

    def test_modify_agents_missing_heartbeat_block_raises(self):
        cfg = {
            "agent_control": {
                "retry": {
                    "attempts": 6,
                    "delay_in_seconds": 60,
                    "max_jitter_in_seconds": 10,
                }
            },
            "agent_script_executor": {
                "bulk_size": 2,
                "bulk_throttle_time_in_ms": 300,
                "indexer_dir_depth": 100,
                "scheduler_cron_time": ["0 */12 * * *"],
            },
            # "heartbeat": missing
        }
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_agents(agent_ids=["agent-123"], config=cfg)

    def test_modify_agents_missing_heartbeat_interval_raises(self):
        cfg = {
            "agent_control": {
                "retry": {
                    "attempts": 6,
                    "delay_in_seconds": 60,
                    "max_jitter_in_seconds": 10,
                }
            },
            "agent_script_executor": {
                "bulk_size": 2,
                "bulk_throttle_time_in_ms": 300,
                "indexer_dir_depth": 100,
                "scheduler_cron_time": ["0 */12 * * *"],
            },
            "heartbeat": {
                # "interval_in_seconds": 300,
                "miss_until_inactive": 1,
            },
        }
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_agents(agent_ids=["agent-123"], config=cfg)

    def test_modify_agents_missing_heartbeat_miss_until_inactive_raises(self):
        cfg = {
            "agent_control": {
                "retry": {
                    "attempts": 6,
                    "delay_in_seconds": 60,
                    "max_jitter_in_seconds": 10,
                }
            },
            "agent_script_executor": {
                "bulk_size": 2,
                "bulk_throttle_time_in_ms": 300,
                "indexer_dir_depth": 100,
                "scheduler_cron_time": ["0 */12 * * *"],
            },
            "heartbeat": {
                "interval_in_seconds": 300,
                # "miss_until_inactive": 1,
            },
        }
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_agents(agent_ids=["agent-123"], config=cfg)

    def test_modify_agents_config_not_mapping_raises(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_agents(
                agent_ids=["agent-123"], config="not-a-mapping"
            )

    def test_modify_agents_agent_control_not_mapping_raises(self):
        cfg = {
            "agent_control": "oops-not-a-mapping",
            "agent_script_executor": {
                "bulk_size": 2,
                "bulk_throttle_time_in_ms": 300,
                "indexer_dir_depth": 100,
                "scheduler_cron_time": ["0 */12 * * *"],
            },
            "heartbeat": {"interval_in_seconds": 300, "miss_until_inactive": 1},
        }
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_agents(agent_ids=["agent-123"], config=cfg)



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224.entities.scan_configs import (
    GmpCloneScanConfigTestMixin,
    GmpCreateScanConfigTestMixin,
    GmpDeleteScanConfigTestMixin,
    GmpGetScanConfigsTestMixin,
    GmpGetScanConfigTestMixin,
    GmpImportScanConfigTestMixin,
    GmpModifyScanConfigSetCommentTestMixin,
    GmpModifyScanConfigSetFamilySelectionTestMixin,
    GmpModifyScanConfigSetNameTestMixin,
    GmpModifyScanConfigSetNvtPreferenceTestMixin,
    GmpModifyScanConfigSetNvtSelectionTestMixin,
    GmpModifyScanConfigSetScannerPreferenceTestMixin,
)
from ...gmpv225 import GMPTestCase


class Gmpv225CloneScanConfigTestCase(GmpCloneScanConfigTestMixin, GMPTestCase):
    pass


class Gmpv225CreateScanConfigTestCase(
    GmpCreateScanConfigTestMixin, GMPTestCase
):
    pass


class Gmpv225DeleteScanConfigTestCase(
    GmpDeleteScanConfigTestMixin, GMPTestCase
):
    pass


class Gmpv225GetScanConfigTestCase(GmpGetScanConfigTestMixin, GMPTestCase):
    pass


class Gmpv225GetScanConfigsTestCase(GmpGetScanConfigsTestMixin, GMPTestCase):
    pass


class Gmpv225ImportScanConfigTestCase(
    GmpImportScanConfigTestMixin, GMPTestCase
):
    pass


class Gmpv225ModifyScanConfigSetCommentTestCase(
    GmpModifyScanConfigSetCommentTestMixin, GMPTestCase
):
    pass


class Gmpv225ModifyScanConfigSetFamilySelectionTestCase(
    GmpModifyScanConfigSetFamilySelectionTestMixin, GMPTestCase
):
    pass


class Gmpv225ModifyScanConfigSetNvtSelectionTestCase(
    GmpModifyScanConfigSetNvtSelectionTestMixin, GMPTestCase
):
    pass


class Gmpv225ModifyScanConfigSetNameTestCase(
    GmpModifyScanConfigSetNameTestMixin, GMPTestCase
):
    pass


class Gmpv225ModifyScanConfigSetNvtPreferenceTestCase(
    GmpModifyScanConfigSetNvtPreferenceTestMixin, GMPTestCase
):
    pass


class Gmpv225ModifyScanConfigSetScannerPreferenceTestCase(
    GmpModifyScanConfigSetScannerPreferenceTestMixin, GMPTestCase
):
    pass



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import RequiredArgument


class GmpModifyGroupTestMixin:
    def test_modify_group(self):
        self.gmp.modify_group(group_id="f1")

        self.connection.send.has_been_called_with(
            b'<modify_group group_id="f1"/>'
        )

    def test_modify_group_missing_group_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_group(group_id=None)

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_group(group_id="")

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_group("")

    def test_modify_group_with_comment(self):
        self.gmp.modify_group(group_id="f1", comment="foo")

        self.connection.send.has_been_called_with(
            b'<modify_group group_id="f1"><comment>foo</comment></modify_group>'
        )

    def test_modify_group_with_name(self):
        self.gmp.modify_group(group_id="f1", name="foo")

        self.connection.send.has_been_called_with(
            b'<modify_group group_id="f1"><name>foo</name></modify_group>'
        )

    def test_modify_group_with_users(self):
        self.gmp.modify_group(group_id="f1", users=["foo"])

        self.connection.send.has_been_called_with(
            b'<modify_group group_id="f1"><users>foo</users></modify_group>'
        )

        self.gmp.modify_group(group_id="f1", users=["foo", "bar"])

        self.connection.send.has_been_called_with(
            b'<modify_group group_id="f1"><users>foo,bar</users></modify_group>'
        )



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv226 import GMPTestCase
from .audit_reports import (
    GmpDeleteAuditReportTestMixin,
    GmpGetAuditReportsTestMixin,
    GmpGetAuditReportTestMixin,
)


class GMPDeleteAuditReportTestCase(GmpDeleteAuditReportTestMixin, GMPTestCase):
    pass


class GMPGetAuditReportTestCase(GmpGetAuditReportTestMixin, GMPTestCase):
    pass


class GMPGetAuditReportsTestCase(GmpGetAuditReportsTestMixin, GMPTestCase):
    pass



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224.entities.filters import (
    GmpCloneFilterTestMixin,
    GmpCreateFilterTestMixin,
    GmpDeleteFilterTestMixin,
    GmpGetFiltersTestMixin,
    GmpGetFilterTestMixin,
    GmpModifyFilterTestMixin,
)
from ...gmpv225 import GMPTestCase


class Gmpv225DeleteFilterTestCase(GmpDeleteFilterTestMixin, GMPTestCase):
    pass


class Gmpv225GetFilterTestCase(GmpGetFilterTestMixin, GMPTestCase):
    pass


class Gmpv225GetFiltersTestCase(GmpGetFiltersTestMixin, GMPTestCase):
    pass


class Gmpv225CloneFilterTestCase(GmpCloneFilterTestMixin, GMPTestCase):
    pass


class Gmpv225CreateFilterTestCase(GmpCreateFilterTestMixin, GMPTestCase):
    pass


class Gmpv225ModifyFilterTestCase(GmpModifyFilterTestMixin, GMPTestCase):
    pass



# SPDX-FileCopyrightText: 2021-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224 import Gmpv224TestCase
from .groups import (
    GmpCloneGroupTestMixin,
    GmpCreateGroupTestMixin,
    GmpDeleteGroupTestMixin,
    GmpGetGroupsTestMixin,
    GmpGetGroupTestMixin,
    GmpModifyGroupTestMixin,
)


class Gmpv224DeleteGroupTestCase(GmpDeleteGroupTestMixin, Gmpv224TestCase):
    pass


class Gmpv224GetGroupTestCase(GmpGetGroupTestMixin, Gmpv224TestCase):
    pass


class Gmpv224GetGroupsTestCase(GmpGetGroupsTestMixin, Gmpv224TestCase):
    pass


class Gmpv224CloneGroupTestCase(GmpCloneGroupTestMixin, Gmpv224TestCase):
    pass


class Gmpv224CreateGroupTestCase(GmpCreateGroupTestMixin, Gmpv224TestCase):
    pass


class Gmpv224ModifyGroupTestCase(GmpModifyGroupTestMixin, Gmpv224TestCase):
    pass



# SPDX-FileCopyrightText: 2022-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import unittest

from gvm.utils import check_port


class TestCheckPort(unittest.TestCase):
    def test_port_cpe(self):
        self.assertTrue(check_port("cpe:meh:moo*"))

    def test_port_cpe_fail(self):
        self.assertFalse(check_port("cpe:meh haha"))

    def test_port_general(self):
        self.assertTrue(check_port("general/tcp"))

    def test_ports(self):
        self.assertTrue(check_port("1/tcp"))
        self.assertTrue(check_port("12/tcp"))
        self.assertTrue(check_port("123/tcp"))
        self.assertTrue(check_port("1234/tcp"))
        self.assertTrue(check_port("12345/tcp"))
        self.assertTrue(check_port("1/udp"))
        self.assertTrue(check_port("12/udp"))

    def test_ports_fail(self):
        self.assertFalse(check_port("1"))
        self.assertFalse(check_port("1//udp"))
        self.assertFalse(check_port("1/12/tcp"))
        self.assertFalse(check_port("tcp"))
        self.assertFalse(check_port("general"))
        self.assertFalse(check_port("lol"))



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224.entities.scanners import (
    GmpCloneScannerTestMixin,
    GmpCreateScannerTestMixin,
    GmpDeleteScannerTestMixin,
    GmpGetScannersTestMixin,
    GmpGetScannerTestMixin,
    GmpModifyScannerTestMixin,
    GmpVerifyScannerTestMixin,
)
from ...gmpv225 import GMPTestCase


class Gmpv225DeleteScannerTestCase(GmpDeleteScannerTestMixin, GMPTestCase):
    pass


class Gmpv225GetScannerTestCase(GmpGetScannerTestMixin, GMPTestCase):
    pass


class Gmpv225GetScannersTestCase(GmpGetScannersTestMixin, GMPTestCase):
    pass


class Gmpv225CloneScannerTestCase(GmpCloneScannerTestMixin, GMPTestCase):
    pass


class Gmpv225CreateScannerTestCase(GmpCreateScannerTestMixin, GMPTestCase):
    pass


class Gmpv225ModifyScannerTestCase(GmpModifyScannerTestMixin, GMPTestCase):
    pass


class Gmpv225VerifyScannerTestCase(GmpVerifyScannerTestMixin, GMPTestCase):
    pass



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224.entities.results import (
    GmpGetResultsTestMixin,
    GmpGetResultTestMixin,
)
from ...gmpv227 import GMPTestCase


class GMPGetResultTestCase(GmpGetResultTestMixin, GMPTestCase):
    pass


class GMPGetResultsTestCase(GmpGetResultsTestMixin, GMPTestCase):
    pass



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest

from gvm.errors import RequiredArgument
from gvm.protocols.gmp.requests.v224 import Vulnerabilities


class VulnerabilitiesTestCase(unittest.TestCase):
    def test_get_vulnerabilities(self):
        request = Vulnerabilities.get_vulnerabilities()
        self.assertEqual(bytes(request), b"<get_vulns/>")

    def test_get_vulnerabilities_with_filter_string(self):
        request = Vulnerabilities.get_vulnerabilities(
            filter_string="filter_string"
        )
        self.assertEqual(
            bytes(request),
            b'<get_vulns filter="filter_string"/>',
        )

    def test_get_vulnerabilities_with_filter_id(self):
        request = Vulnerabilities.get_vulnerabilities(filter_id="filter_id")
        self.assertEqual(
            bytes(request),
            b'<get_vulns filt_id="filter_id"/>',
        )

    def test_get_vulnerability(self):
        request = Vulnerabilities.get_vulnerability("vulnerability_id")
        self.assertEqual(
            bytes(request),
            b'<get_vulns vuln_id="vulnerability_id"/>',
        )

    def test_get_vulnerability_missing_vulnerability_id(self):
        with self.assertRaises(RequiredArgument):
            Vulnerabilities.get_vulnerability(None)

        with self.assertRaises(RequiredArgument):
            Vulnerabilities.get_vulnerability("")



# SPDX-FileCopyrightText: 2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later


from gvm.errors import RequiredArgument
from gvm.protocols.core import Request
from gvm.utils import to_bool, to_comma_list
from gvm.xml import XmlCommand

from .._entity_id import EntityID


class Groups:
    @classmethod
    def clone_group(cls, group_id: EntityID) -> Request:
        """Clone an existing group

        Args:
            group_id: UUID of an existing group to clone from
        """
        if not group_id:
            raise RequiredArgument(
                function=cls.clone_group.__name__, argument="group_id"
            )

        cmd = XmlCommand("create_group")
        cmd.add_element("copy", str(group_id))

        return cmd

    @classmethod
    def create_group(
        cls,
        name: str,
        *,
        comment: str | None = None,
        special: bool | None = False,
        users: list[str] | None = None,
    ) -> Request:
        """Create a new group

        Args:
            name: Name of the new group
            comment: Comment for the group
            special: Create permission giving members full access to each
                other's entities
            users: List of user names to be in the group
        """
        if not name:
            raise RequiredArgument(
                function=cls.create_group.__name__, argument="name"
            )

        cmd = XmlCommand("create_group")
        cmd.add_element("name", name)

        if comment:
            cmd.add_element("comment", comment)

        if special:
            xml_special = cmd.add_element("specials")
            xml_special.add_element("full")

        if users:
            cmd.add_element("users", to_comma_list(users))

        return cmd

    @classmethod
    def delete_group(
        cls, group_id: EntityID, *, ultimate: bool | None = False
    ) -> Request:
        """Deletes an existing group

        Args:
            group_id: UUID of the group to be deleted.
            ultimate: Whether to remove entirely, or to the trashcan.
        """
        if not group_id:
            raise RequiredArgument(
                function=cls.delete_group.__name__, argument="group_id"
            )

        cmd = XmlCommand("delete_group")
        cmd.set_attribute("group_id", str(group_id))
        cmd.set_attribute("ultimate", to_bool(ultimate))

        return cmd

    @staticmethod
    def get_groups(
        *,
        filter_string: str | None = None,
        filter_id: EntityID | None = None,
        trash: bool | None = None,
    ) -> Request:
        """Request a list of groups

        Args:
            filter_string: Filter term to use for the query
            filter_id: UUID of an existing filter to use for the query
            trash: Whether to get the trashcan groups instead
        """
        cmd = XmlCommand("get_groups")

        cmd.add_filter(filter_string, filter_id)

        if trash is not None:
            cmd.set_attribute("trash", to_bool(trash))

        return cmd

    @classmethod
    def get_group(cls, group_id: EntityID) -> Request:
        """Request a single group

        Args:
            group_id: UUID of an existing group
        """
        cmd = XmlCommand("get_groups")

        if not group_id:
            raise RequiredArgument(
                function=cls.get_group.__name__, argument="group_id"
            )

        cmd.set_attribute("group_id", str(group_id))
        return cmd

    @classmethod
    def modify_group(
        cls,
        group_id: EntityID,
        *,
        comment: str | None = None,
        name: str | None = None,
        users: list[str] | None = None,
    ) -> Request:
        """Modifies an existing group.

        Args:
            group_id: UUID of group to modify.
            comment: Comment on group.
            name: Name of group.
            users: List of user names to be in the group
        """
        if not group_id:
            raise RequiredArgument(
                function=cls.modify_group.__name__, argument="group_id"
            )

        cmd = XmlCommand("modify_group")
        cmd.set_attribute("group_id", str(group_id))

        if comment:
            cmd.add_element("comment", comment)

        if name:
            cmd.add_element("name", name)

        if users:
            cmd.add_element("users", to_comma_list(users))

        return cmd



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#


class GmpDescribeAuthTestMixin:
    def test_describe_auth(self):
        self.gmp.describe_auth()

        self.connection.send.has_been_called_with(b"<describe_auth/>")



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import RequiredArgument


class GmpGetResultTestMixin:
    def test_get_result(self):
        self.gmp.get_result("r1")

        self.connection.send.has_been_called_with(
            b'<get_results result_id="r1" details="1"/>'
        )

        self.gmp.get_result(result_id="r1")

        self.connection.send.has_been_called_with(
            b'<get_results result_id="r1" details="1"/>'
        )

    def test_get_result_missing_result_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.get_result(result_id=None)

        with self.assertRaises(RequiredArgument):
            self.gmp.get_result("")



# SPDX-FileCopyrightText: 2023-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from ...gmpv224.entities.schedules import (
    GmpCloneScheduleTestMixin,
    GmpCreateScheduleTestMixin,
    GmpDeleteScheduleTestMixin,
    GmpGetSchedulesTestMixin,
    GmpGetScheduleTestMixin,
    GmpModifyScheduleTestMixin,
)
from ...gmpv227 import GMPTestCase


class GMPDeleteScheduleTestCase(GmpDeleteScheduleTestMixin, GMPTestCase):
    pass


class GMPGetScheduleTestCase(GmpGetScheduleTestMixin, GMPTestCase):
    pass


class GMPGetSchedulesTestCase(GmpGetSchedulesTestMixin, GMPTestCase):
    pass


class GMPCloneScheduleTestCase(GmpCloneScheduleTestMixin, GMPTestCase):
    pass


class GMPCreateScheduleTestCase(GmpCreateScheduleTestMixin, GMPTestCase):
    pass


class GMPModifyScheduleTestCase(GmpModifyScheduleTestMixin, GMPTestCase):
    pass



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import RequiredArgument


class GmpGetScanConfigNvtTestMixin:
    def test_get_scan_config_nvt_with_nvt_oid(self):
        self.gmp.get_scan_config_nvt(nvt_oid="nvt_oid")

        self.connection.send.has_been_called_with(
            b'<get_nvts nvt_oid="nvt_oid" details="1" '
            b'preferences="1" preference_count="1"/>'
        )

    def test_get_scan_config_nvt_missing_nvt_oid(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.get_scan_config_nvt(nvt_oid=None)

        with self.assertRaises(RequiredArgument):
            self.gmp.get_scan_config_nvt(nvt_oid="")

        with self.assertRaises(RequiredArgument):
            self.gmp.get_scan_config_nvt("")



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#


class GmpGetNvtFamiliesTestMixin:
    def test_get_nvt_families(self):
        self.gmp.get_nvt_families()

        self.connection.send.has_been_called_with(b"<get_nvt_families/>")

    def test_get_nvt_families_with_sort_order(self):
        self.gmp.get_nvt_families(sort_order="foo")

        self.connection.send.has_been_called_with(
            b'<get_nvt_families sort_order="foo"/>'
        )



# SPDX-FileCopyrightText: 2018-2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import GvmError


class GmpStopTaskTestMixin:
    def test_stop_task(self):
        self.gmp.stop_task("a1")

        self.connection.send.has_been_called_with(b'<stop_task task_id="a1"/>')

    def test_missing_id(self):
        with self.assertRaises(GvmError):
            self.gmp.stop_task(None)

        with self.assertRaises(GvmError):
            self.gmp.stop_task("")



# SPDX-FileCopyrightText: 2018-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from gvm.errors import InvalidArgument, InvalidArgumentType, RequiredArgument


class GmpModifyScanConfigSetFamilySelectionTestMixin:
    def test_modify_scan_config_set_family_selection(self):
        self.gmp.modify_scan_config_set_family_selection(
            config_id="c1", families=[("foo", True, True)]
        )

        self.connection.send.has_been_called_with(
            b'<modify_config config_id="c1">'
            b"<family_selection>"
            b"<growing>1</growing>"
            b"<family>"
            b"<name>foo</name>"
            b"<all>1</all>"
            b"<growing>1</growing>"
            b"</family>"
            b"</family_selection>"
            b"</modify_config>"
        )

        self.gmp.modify_scan_config_set_family_selection(
            config_id="c1", families=[("foo", True, True), ("bar", True, True)]
        )

        self.connection.send.has_been_called_with(
            b'<modify_config config_id="c1">'
            b"<family_selection>"
            b"<growing>1</growing>"
            b"<family>"
            b"<name>foo</name>"
            b"<all>1</all>"
            b"<growing>1</growing>"
            b"</family>"
            b"<family>"
            b"<name>bar</name>"
            b"<all>1</all>"
            b"<growing>1</growing>"
            b"</family>"
            b"</family_selection>"
            b"</modify_config>"
        )

        self.gmp.modify_scan_config_set_family_selection(
            config_id="c1", families=(("foo", True, True), ("bar", True, True))
        )

        self.connection.send.has_been_called_with(
            b'<modify_config config_id="c1">'
            b"<family_selection>"
            b"<growing>1</growing>"
            b"<family>"
            b"<name>foo</name>"
            b"<all>1</all>"
            b"<growing>1</growing>"
            b"</family>"
            b"<family>"
            b"<name>bar</name>"
            b"<all>1</all>"
            b"<growing>1</growing>"
            b"</family>"
            b"</family_selection>"
            b"</modify_config>"
        )

        self.gmp.modify_scan_config_set_family_selection(
            config_id="c1",
            families=[("foo", True, False), ("bar", False, True)],
        )

        self.connection.send.has_been_called_with(
            b'<modify_config config_id="c1">'
            b"<family_selection>"
            b"<growing>1</growing>"
            b"<family>"
            b"<name>foo</name>"
            b"<all>0</all>"
            b"<growing>1</growing>"
            b"</family>"
            b"<family>"
            b"<name>bar</name>"
            b"<all>1</all>"
            b"<growing>0</growing>"
            b"</family>"
            b"</family_selection>"
            b"</modify_config>"
        )

    def test_modify_scan_config_set_family_selection_missing_config_id(self):
        with self.assertRaises(RequiredArgument):
            self.gmp.modify_scan_config_set_family_selection(
                config_id=None, families=[("foo", True, True)]
            )

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_scan_config_set_family_selection(
                config_id="", families=[("foo", True, True)]
            )

        with self.assertRaises(RequiredArgument):
            self.gmp.modify_scan_config_set_family_selection(
                "", [("foo", True)]
            )

    def test_modify_scan_config_set_family_selection_invalid_families(self):
        with self.assertRaises(InvalidArgumentType):
            self.gmp.modify_scan_config_set_family_selection(
                config_id="c1", families=None
            )

        with self.assertRaises(InvalidArgumentType):
            self.gmp.modify_scan_config_set_family_selection(
                config_id="c1", families=""
            )

        with self.assertRaises(InvalidArgumentType):
            self.gmp.modify_scan_config_set_family_selection("c1", "")

    def test_modify_scan_config_set_family_selection_with_auto_add_new_families(
        self,
    ):
        self.gmp.modify_scan_config_set_family_selection(
            config_id="c1",
            families=[("foo", True, True)],
            auto_add_new_families=True,
        )

        self.connection.send.has_been_called_with(
            b'<modify_config config_id="c1">'
            b"<family_selection>"
            b"<growing>1</growing>"
            b"<family>"
            b"<name>foo</name>"
            b"<all>1</all>"
            b"<growing>1</growing>"
            b"</family>"
            b"</family_selection>"
            b"</modify_config>"
        )

        self.gmp.modify_scan_config_set_family_selection(
            config_id="c1",
            families=[("foo", True, True)],
            auto_add_new_families=False,
        )

        self.connection.send.has_been_called_with(
            b'<modify_config config_id="c1">'
            b"<family_selection>"
            b"<growing>0</growing>"
            b"<family>"
            b"<name>foo</name>"
            b"<all>1</all>"
            b"<growing>1</growing>"
            b"</family>"
            b"</family_selection>"
            b"</modify_config>"
        )

    def test_modify_scan_config_set_family_selection_with_auto_add_new_nvts(
        self,
    ):
        self.gmp.modify_scan_config_set_family_selection(
            config_id="c1", families=[("foo", True, True)]
        )

        self.connection.send.has_been_called_with(
            b'<modify_config config_id="c1">'
            b"<family_selection>"
            b"<growing>1</growing>"
            b"<family>"
            b"<name>foo</name>"
            b"<all>1</all>"
            b"<growing>1</growing>"
            b"</family>"
            b"</family_selection>"
            b"</modify_config>"
        )

        self.gmp.modify_scan_config_set_family_selection(
            config_id="c1", families=[("foo", False, True)]
        )

        self.connection.send.has_been_called_with(
            b'<modify_config config_id="c1">'
            b"<family_selection>"
            b"<growing>1</growing>"
            b"<family>"
            b"<name>foo</name>"
            b"<all>1</all>"
            b"<growing>0</growing>"
            b"</family>"
            b"</family_selection>"
            b"</modify_config>"
        )

        self.gmp.modify_scan_config_set_family_selection(
            config_id="c1",
            families=[("foo", False, True), ("bar", True, False)],
        )

        self.connection.send.has_been_called_with(
            b'<modify_config config_id="c1">'
            b"<family_selection>"
            b"<growing>1</growing>"
            b"<family>"
            b"<name>foo</name>"
            b"<all>1</all>"
            b"<growing>0</growing>"
            b"</family>"
            b"<family>"
            b"<name>bar</name>"
            b"<all>0</all>"
            b"<growing>1</growing>"
            b"</family>"
            b"</family_selection>"
            b"</modify_config>"
        )

        with self.assertRaises(InvalidArgumentType):
            self.gmp.modify_scan_config_set_family_selection(
                config_id="c1", families=[("foo", "False", "True")]
            )

        with self.assertRaises(InvalidArgumentType):
            self.gmp.modify_scan_config_set_family_selection(
                config_id="c1", families=[("foo", True, None)]
            )

        with self.assertRaises(InvalidArgumentType):
            self.gmp.modify_scan_config_set_family_selection(
                config_id="c1", families=[("foo", "True", False)]
            )

        with self.assertRaises(InvalidArgument):
            self.gmp.modify_scan_config_set_family_selection(
                config_id="c1", families=[("foo",)]
            )



# SPDX-FileCopyrightText: 2025 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later

import unittest

from gvm.errors import RequiredArgument
from gvm.protocols.gmp.requests.v226 import ReportConfigs, ReportFormatType
from gvm.protocols.gmp.requests.v226._report_configs import (
    ReportConfigParameter,
)


class ReportConfigsTestCase(unittest.TestCase):
    def test_create_report_config(self):
        request = ReportConfigs.create_report_config(
            "report-config1", "report_format_id"
        )
        self.assertEqual(
            bytes(request),
            b"<create_report_config>"
            b"<name>report-config1</name>"
            b'<report_format id="report_format_id"/>'
            b"</create_report_config>",
        )

    def test_create_report_config_with_comment(self):
        request = ReportConfigs.create_report_config(
            "report-config1", "report_format_id", comment="foo"
        )
        self.assertEqual(
            bytes(request),
            b"<create_report_config>"
            b"<name>report-config1</name>"
            b'<report_format id="report_format_id"/>'
            b"<comment>foo</comment>"
            b"</create_report_config>",
        )

    def test_create_report_config_with_params(self):
        request = ReportConfigs.create_report_config(
            "report-config1",
            "report_format_id",
            params=[
                ReportConfigParameter("name", "value"),
                ReportConfigParameter("name2", "value2", use_default=True),
                ReportConfigParameter("name3", use_default=True),
            ],
        )
        self.assertEqual(
            bytes(request),
            b"<create_report_config>"
            b"<name>report-config1</name>"
            b'<report_format id="report_format_id"/>'
            b'<param><name>name</name><value use_default="0">value</value></param>'
            b'<param><name>name2</name><value use_default="1"/></param>'
            b'<param><name>name3</name><value use_default="1"/></param>'
            b"</create_report_config>",
        )

    def test_create_report_config_missing_name(self):
        with self.assertRaises(RequiredArgument):
            ReportConfigs.create_report_config(None, "report_format_id")

        with self.assertRaises(RequiredArgument):
            ReportConfigs.create_report_config("", "report_format_id")

    def test_create_report_config_missing_report_format_id(self):
        with self.assertRaises(RequiredArgument):
            ReportConfigs.create_report_config("foo", None)

        with self.assertRaises(RequiredArgument):
            ReportConfigs.create_report_config("foo", "")

    def test_clone_report_config(self):
        request = ReportConfigs.clone_report_config("report_config_id")
        self.assertEqual(
            bytes(request),
            b"<create_report_config>"
            b"<copy>report_config_id</copy>"
            b"</create_report_config>",
        )

        request = ReportConfigs.clone_report_config(ReportFormatType.PDF)
        self.assertEqual(
            bytes(request),
            b"<create_report_config>"
            b"<copy>c402cc3e-b531-11e1-9163-406186ea4fc5</copy>"
            b"</create_report_config>",
        )

    def test_clone_report_config_missing_report_config_id(self):
        with self.assertRaises(RequiredArgument):
            ReportConfigs.clone_report_config(None)

        with self.assertRaises(RequiredArgument):
            ReportConfigs.clone_report_config("")

    def test_delete_report_config(self):
        request = ReportConfigs.delete_report_config("report_config_id")
        self.assertEqual(
            bytes(request),
            b'<delete_report_config report_config_id="report_config_id" ultimate="0"/>',
        )

        request = ReportConfigs.delete_report_config(ReportFormatType.PDF)
        self.assertEqual(
            bytes(request),
            b'<delete_report_config report_config_id="c402cc3e-b531-11e1-9163-406186ea4fc5" ultimate="0"/>',
        )

    def test_delete_report_config_with_ultimate(self):
        request = ReportConfigs.delete_report_config(
            "report_config_id", ultimate=True
        )
        self.assertEqual(
            bytes(request),
            b'<delete_report_config report_config_id="report_config_id" ultimate="1"/>',
        )

        request = ReportConfigs.delete_report_config(
            ReportFormatType.PDF, ultimate=False
        )
        self.assertEqual(
            bytes(request),
            b'<delete_report_config report_config_id="c402cc3e-b531-11e1-9163-406186ea4fc5" ultimate="0"/>',
        )

    def test_delete_report_config_missing_report_config_id(self):
        with self.assertRaises(RequiredArgument):
            ReportConfigs.delete_report_config(None)

        with self.assertRaises(RequiredArgument):
            ReportConfigs.delete_report_config("")

    def test_get_report_configs(self):
        request = ReportConfigs.get_report_configs()
        self.assertEqual(
            bytes(request),
            b"<get_report_configs/>",
        )

    def test_get_report_configs_with_filter_string(self):
        request = ReportConfigs.get_report_configs(
            filter_string="filter_string"
        )
        self.assertEqual(
            bytes(request),
            b'<get_report_configs filter="filter_string"/>',
        )

    def test_get_report_configs_with_filter_id(self):
        request = ReportConfigs.get_report_configs(filter_id="filter_id")
        self.assertEqual(
            bytes(request),
            b'<get_report_configs filt_id="filter_id"/>',
        )

    def test_get_report_configs_with_details(self):
        request = ReportConfigs.get_report_configs(details=True)
        self.assertEqual(
            bytes(request),
            b'<get_report_configs details="1"/>',
        )

        request = ReportConfigs.get_report_configs(details=False)
        self.assertEqual(
            bytes(request),
            b'<get_report_configs details="0"/>',
        )

    def test_get_report_configs_with_trash(self):
        request = ReportConfigs.get_report_configs(trash=True)
        self.assertEqual(
            bytes(request),
            b'<get_report_configs trash="1"/>',
        )

        request = ReportConfigs.get_report_configs(trash=False)
        self.assertEqual(
            bytes(request),
            b'<get_report_configs trash="0"/>',
        )

    def test_get_report_config(self):
        request = ReportConfigs.get_report_config("report_config_id")
        self.assertEqual(
            bytes(request),
            b'<get_report_configs report_config_id="report_config_id" details="1"/>',
        )

    def test_get_report_config_missing_report_config_id(self):
        with self.assertRaises(RequiredArgument):
            ReportConfigs.get_report_config(None)

        with self.assertRaises(RequiredArgument):
            ReportConfigs.get_report_config("")

    def test_modify_report_config(self):
        request = ReportConfigs.modify_report_config("report_config_id")
        self.assertEqual(
            bytes(request),
            b'<modify_report_config report_config_id="report_config_id"/>',
        )

    def test_modify_report_config_with_name(self):
        request = ReportConfigs.modify_report_config(
            "report_config_id", name="foo"
        )
        self.assertEqual(
            bytes(request),
            b'<modify_report_config report_config_id="report_config_id">'
            b"<name>foo</name>"
            b"</modify_report_config>",
        )

    def test_modify_report_config_with_comment(self):
        request = ReportConfigs.modify_report_config(
            "report_config_id", comment="foo"
        )
        self.assertEqual(
            bytes(request),
            b'<modify_report_config report_config_id="report_config_id">'
            b"<comment>foo</comment>"
            b"</modify_report_config>",
        )

    def test_modify_report_config_with_params(self):
        request = ReportConfigs.modify_report_config(
            "report_config_id",
            params=[
                ReportConfigParameter("name", "value"),
                ReportConfigParameter("name2", "value2", use_default=True),
                ReportConfigParameter("name3", use_default=True),
            ],
        )
        self.assertEqual(
            bytes(request),
            b'<modify_report_config report_config_id="report_config_id">'
            b'<param><name>name</name><value use_default="0">value</value></param>'
            b'<param><name>name2</name><value use_default="1"/></param>'
            b'<param><name>name3</name><value use_default="1"/></param>'
            b"</modify_report_config>",
        )

    def test_modify_report_config_missing_report_config_id(self):
        with self.assertRaises(RequiredArgument):
            ReportConfigs.modify_report_config(None)

        with self.assertRaises(RequiredArgument):
            ReportConfigs.modify_report_config("")
